/* Next available MSG number is  88 */

/*
    ADS_PERR.C
      Copyright (C) 1988-1993 by Autodesk, Inc.

      Permission to use, copy, modify, and distribute this software
      for any purpose and without fee is hereby granted, provided
      that the above copyright notice appears in all copies and that
      both that copyright notice and this permission notice appear in
      all supporting documentation.

      THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED
      WARRANTY.  ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR
      PURPOSE AND OF MERCHANTABILITY ARE HEREBY DISCLAIMED.


    ADS_PERR.C	-  ADS routine to print the error message
                   associated with the value of "errno" upon failure
		   of a call to AutoCAD.  This routine is meant to be
		   invoked by other ADS programs, and contains a table
		   of the current error messages as found in ol_errno.h.
		   This routine is meant to parallel the Unix
		   perror() function.

		   ads_perror() is a Lisp-callable or invokable function that
		   takes an optional single argument and prints the message on
		   the AutoCAD command line.  The syntax is:

                   (ads_perror "string")

		   The string is optional; if provided, it is printed on a
		   separate line before the system error message.  To be of
		   most use, the argument string should include the name of the
		   program that incurred the error.  The error number is taken
                   from the system variable "errno", which is set when most
		   errors occur, but not cleared when non-erroneous calls are
		   made.

		   The STANDALONE macro is obsolete.  Forget it.
 */

#include  <stdio.h>
#include  <string.h>
#include  "adslib.h"
#include  "ol_errno.h"


/* The error message table */

char *errmsg[] = {					       /* ERRNO code */
    /*MSG1*/"Invalid symbol table name.",                              /*  1 */
    /*MSG2*/"Invalid name (entity or selection set) value.",           /*  2 */
    /*MSG3*/"Exceeded max number of selection sets.",                  /*  3 */
    /*MSG4*/"Invalid selection set.",                                  /*  4 */
    /*MSG5*/"Improper use of Block Definition entity.",                /*  5 */
    /*MSG6*/"Improper use of Xref entity.",                            /*  6 */
    /*MSG7*/"Entity selection failure by pick.",                       /*  7 */
    /*MSG8*/"End of entity file.",                                     /*  8 */
    /*MSG9*/"End of Block Definition file.",                           /*  9 */
    /*MSG10*/"Failure of eidlast.",                                    /* 10 */
    /*MSG11*/"Illegal entdel of Viewport.",                            /* 11 */
    /*MSG12*/"Operation not allowed during PLINE.",                    /* 12 */
    /*MSG13*/"Invalid handle.",                                        /* 13 */
    /*MSG14*/"Handles not enabled.",                                   /* 14 */
    /*MSG15*/"Invalid TRANS request.",                                 /* 15 */
    /*MSG16*/"Invalid space trans request.",                           /* 16 */
    /*MSG17*/"Invalid use of deleted entity.",                         /* 17 */
    /*MSG18*/"Invalid table name.",                                    /* 18 */
    /*MSG19*/"Invalid table function argument.",                       /* 19 */
    /*MSG20*/"Attempt to set read-only variable.",                     /* 20 */
    /*MSG21*/"Zero value invalid.",                                    /* 21 */
    /*MSG22*/"Value out of range.",                                    /* 22 */
    /*MSG23*/"Complex regen in progress.",                             /* 23 */
    /*MSG24*/"Attempt to change entity type.",                         /* 24 */
    /*MSG25*/"Bad layer name.",                                        /* 25 */
    /*MSG26*/"Bad linetype name.",                                     /* 26 */
    /*MSG27*/"Bad color name.",                                        /* 27 */
    /*MSG28*/"Bad text style name.",                                   /* 28 */
    /*MSG29*/"Bad shape name.",                                        /* 29 */
    /*MSG30*/"Bad field for entity type.",                             /* 30 */
    /*MSG31*/"Attempted entmod of deleted entity.",                    /* 31 */
    /*MSG32*/"Attempted entmod of SEQEND.",                            /* 32 */
    /*MSG33*/"Attempt to change handle.",                              /* 33 */
    /*MSG34*/"Illegal modification of viewport visibility.",           /* 34 */
    /*MSG35*/"Entity on locked layer.",                                /* 35 */
    /*MSG36*/"Bad entity type.",                                       /* 36 */
    /*MSG37*/"Bad PLINE entity.",                                      /* 37 */
    /*MSG38*/"Incomplete complex entity in block.",                    /* 38 */
    /*MSG39*/"Invalid block name field.",                              /* 39 */
    /*MSG40*/"Duplicate block flag fields.",                           /* 40 */
    /*MSG41*/"Duplicate block name fields.",                           /* 41 */
    /*MSG42*/"Bad normal vector.",                                     /* 42 */
    /*MSG43*/"Missing block name.",                                    /* 43 */
    /*MSG44*/"Missing block flags.",                                   /* 44 */
    /*MSG45*/"Invalid anonymous block.",                               /* 45 */
    /*MSG46*/"Invalid Block Definition entity.",                       /* 46 */
    /*MSG47*/"Mandatory field missing.",                               /* 47 */
    /*MSG48*/"Unrecognized extended data type.",                       /* 48 */
    /*MSG49*/"Improper nesting of list in Xdata.",                     /* 49 */
    /*MSG50*/"Improper location of APPID field.",                      /* 50 */
    /*MSG51*/"Exceeded maximum Xdata size.",                           /* 51 */
    /*MSG52*/"Entity selection failure by null response.",             /* 52 */
    /*MSG53*/"Duplicate application name in Xdata.",                   /* 53 */
    /*MSG56*/"Attempt to make or modify Viewport entity.",             /* 54 */
    /*MSG57*/"Attempt to make an Xref or dependent symbol.",           /* 55 */
    /*MSG58*/"Bad ssget filter: unterminated clause.",                 /* 56 */
    /*MSG59*/"Bad ssget filter: missing test operand.",                /* 57 */
    /*MSG60*/"Bad ssget filter: invalid test operation string.",       /* 58 */
    /*MSG61*/"Bad ssget filter: empty clause or improper nesting.",    /* 59 */
    /*MSG62*/"Bad ssget filter: begin/end clause mismatch.",           /* 60 */
    /*MSG63*/"Bad ssget filter: wrong number of XOR/NOT operands.",    /* 61 */
    /*MSG64*/"Bad ssget filter: maximum nesting level exceeded.",      /* 62 */
    /*MSG65*/"Bad ssget filter: invalid group code.",                  /* 63 */
    /*MSG66*/"Bad ssget filter: invalid string test.",                 /* 64 */
    /*MSG67*/"Bad ssget filter: invalid vector test.",                 /* 65 */
    /*MSG68*/"Bad ssget filter: invalid real test.",                   /* 66 */
    /*MSG69*/"Bad ssget filter: invalid integer test.",                /* 67 */
    /*MSG70*/"Digitizer isn't a tablet",                               /* 68 */
    /*MSG71*/"Tablet isn't calibrated.",                               /* 69 */
    /*MSG72*/"Invalid arguments to (TABLET) function.",                /* 70 */
    /*MSG73*/"Not enough memory to allocate resbuf.",                  /* 71 */
    /*MSG74*/"NULL Pointer was provided as an argument.",              /* 72 */
    /*MSG75*/"The specified file can't be opened.",                    /* 73 */
    /*MSG76*/"The specified application is already loaded.",           /* 74 */
    /*MSG77*/"The maximum number of loaded ADS apps has been reached.",/* 75 */
    /*MSG78*/"The specified application could not be executed.",       /* 76 */
    /*MSG79*/"The ADS app has an incompatible version number.",        /* 77 */
    /*MSG80*/"The ADS app is active or nested and can't be unloaded.", /* 78 */
    /*MSG81*/"The ADS application refused to XUNLOAD.",                /* 79 */
    /*MSG82*/"The specified ADS application is not loaded.",           /* 80 */
    /*MSG83*/"Insuficient memory to load ADS application.",            /* 81 */
    /*MSG84*/"Invalid transformation matrix.",                         /* 82 */
    /*MSG85*/"Invalid symbol name.",                                   /* 83 */
    /*MSG86*/"Invalid symbol value.",                                  /* 84 */
    /*MSG87*/"Operation not allowed while a dialogue box is active.",  /* 85 */
   };

/*  Local function declarations */

int	ads_perror  _((void));

#define STANDALONE 1		      /* This is meant to be ads_invoked */

#ifdef STANDALONE

void	main	    _((int, char **));
int	loadfuncs   _((void));


/* MAIN -- the main routine */
void
main(argc,argv)
  int argc;
  char *argv[];
{
    int stat;
    short scode = RSRSLT;
    char errmsg[80];

    ads_init(argc, argv);

    for ( ;; ) {

	if ((stat = ads_link(scode)) < 0) {
            sprintf(errmsg, /*MSG54*/"ADS_PERR: bad status from ads_link() = %d\n",
		   stat);
#ifdef Macintosh
	    macalert(errmsg);
#else
	    printf("%s",errmsg);
#endif /* Macintosh */
	    exit(1);
	}

	scode = RSRSLT; 	      /* default return value */

	/* Check for AT LEAST the following cases here */
	switch (stat) {

	case RQXLOAD:
	    scode = loadfuncs() ? RSRSLT : -RSERR;
	    break;

	case RQSUBR:
	    ads_perror();
	    break;

	default:
	    break;

	}
    }
}


/* LOADFUNCS  --  Define external functions with AutoLISP */
static int loadfuncs()
{
    if (!ads_defun(/*MSG0*/"ads_perror", 0))
	return FALSE;

    /* Although this function is never involved in some recursive call,
       we ads_regfunc it so that future implementations may do away
       with the need to go through the dispatch loop.
    */

    ads_regfunc(ads_perror, 0);
    return 1;
}

#endif	/* STANDALONE */

#ifdef STANDALONE
static
#endif
int
/*FCN*/ads_perror()
{
    struct resbuf *argl, errval;

    argl = ads_getargs();

    if (ads_getfuncode() != 0)	      /* This is the ONLY function here */
	return RSRSLT;

    if (ads_getvar(/*MSG0*/"ERRNO", &errval) == RTERROR)
	return RSRSLT;

    if (errval.resval.rint < 1 || errval.resval.rint > MAX_OL_ERRNO)
	return RSRSLT;

    if (argl && argl->restype == RTSTR && strlen(argl->resval.rstring))
        ads_printf("%s: ", argl->resval.rstring);

    ads_printf("%s\n", errmsg[errval.resval.rint - 1]);

    ads_relrb(argl);
    ads_retvoid();

    return RSRSLT;
}
