/* Next available MSG number is  66 */

/*****************************************************************************
      CALERR.C
      (C) Copyright 1988-1993 by Autodesk, Inc.

      This program is copyrighted by Autodesk, Inc. and is  licensed
      to you under the following conditions.  You may not distribute
      or  publish the source code of this program in any form.   You
      may  incorporate this code in object form in derivative  works
      provided  such  derivative  works  are  (i.) are  designed and 
      intended  to  work  solely  with  Autodesk, Inc. products, and 
      (ii.)  contain  Autodesk's  copyright  notice  "(C)  Copyright  
      1988-1993 by Autodesk, Inc."

      AUTODESK  PROVIDES THIS PROGRAM "AS IS" AND WITH  ALL  FAULTS.
      AUTODESK  SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF  MER-
      CHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK,  INC.
      DOES  NOT  WARRANT THAT THE OPERATION OF THE PROGRAM  WILL  BE
      UNINTERRUPTED OR ERROR FREE.

  Description: Error reporting function for the Geometry Calculator ADS
               application.

*****************************************************************************/


/****************************************************************************/
/*  INCLUDES                                                                */
/****************************************************************************/

#include "cal.h"


/****************************************************************************/
/*  EXPORTED VARIABLES                                                      */
/****************************************************************************/

int cal_err;                          /* Error number, 0 means O.K. */


/****************************************************************************/
/*  STATIC VARIABLES                                                        */
/****************************************************************************/

static char *error_messages[] =
{
   "%s",                                                            /* 0*/
   /*MSG1*/"Wrong real number '%s'",                                        /* 1*/
   /*MSG2*/"Illegal character '%s'",
   /*MSG3*/"Integer numbers must be between -32768 and 32767",
   /*MSG4*/"Function %s requires one argument",
   /*MSG5*/"Right parenthesis ')' missing",                                 /* 5*/
   /*MSG6*/"Vector components must be real numbers",
   /*MSG7*/"Wrong syntax of vector, maybe ',' missing",
   /*MSG8*/"Wrong syntax of vector, maybe ']' missing",
   /*MSG9*/"Unexpected end of expression",
   /*MSG10*/"Wrong syntax of expression",                                    /*10*/
   /*MSG11*/"Function %s requires vector argument",
   /*MSG12*/"Function %s requires real argument",
   /*MSG13*/"Argument to ASIN, ACOS must be between -1 and 1",
   /*MSG14*/"Argument to ROUND and TRUNC must be between -32768 and 32767",
   /*MSG15*/"Argument to SQRT cannot be negative",                           /*15*/
   /*MSG16*/"Power operator '^' requires real operands",
   /*MSG17*/"Negative numbers cannot be raised to real exponent",
   /*MSG18*/"Division by 0",
   /*MSG19*/"Operator '/' requires real-real or vector-real operands",
   /*MSG20*/"Operator '&' requires vector operands",                         /*20*/
   /*MSG21*/"Operators '+', '-' require operands of the same type",
   /*MSG22*/"Invalid feet'-inches\" format",
   /*MSG23*/"Numeric overflow",
   /*MSG24*/"Function TANG does not accept arguments 90+k*180 degrees",
   /*MSG25*/"Function %s takes no arguments",                                /*25*/
   /*MSG26*/"Use %s",
   /*MSG27*/"The calculator expression must be a string",
   /*MSG28*/"Assignment operator '=' used improperly",
   /*MSG29*/"Failed to assign the value of the expression to variable %s",
   /*MSG30*/"The 2D vector for NOR(v) function has zero length",             /*30*/
   /*MSG31*/"AutoLISP symbol '%s ...' is too long",
   /*MSG32*/"Unknown AutoCAD system variable %s",
   /*MSG33*/"Only real, integer or point AutoCAD system variables accepted",
   /*MSG34*/"Angles should be written in <deg>d<min>'<sec>\" format",
   /*MSG35*/"Wrong syntax of point (vector)",                                /*35*/
   /*MSG36*/"Point 'p' in ANG(apex,p1,p2,p) function lies on plane (apex,p1,p2)",
   /*MSG37*/"Points in ANG(apex,p1,p2,p) function coincide",
   /*MSG38*/"Function %s requires positive argument",
   /*MSG39*/"The 2D points in ANG(p1,p2) function coincide",
   /*MSG40*/"The two points defining line in function %s coincide",          /*40*/
   /*MSG41*/"Integer value of expression is not between -32768 and 32767",
   /*MSG42*/"The function name %s is not an identifier",
   /*MSG43*/"Value of AutoLISP variable %s must be a point, real or integer",
   /*MSG44*/"Too many calculator functions",
   /*MSG45*/"AutoLISP variable %s has no value",                             /*45*/
   /*MSG46*/"Null function name or function pointer to cal_register_function()",
   /*MSG47*/"Function name %s is too long",
   /*MSG48*/"The point to be rotated lies on the axis of rotation",
   /*MSG49*/"Comma expected in argument list of function %s",
   /*MSG50*/"Too many arguments to function %s",                             /*50*/
   /*MSG51*/"The points in NOR(p1,p2,p3) function coincide or are collinear",
   /*MSG52*/"Function %s doesn't exist",
   /*MSG53*/"The points 'p1ax' and 'p2ax' in ROT(p,p1ax,p2ax,angle) coincide",
   /*MSG54*/"The two lines in function %s do not intersect or are collinear",
   /*MSG55*/"The line is parallel to the plane",                             /*55*/
   /*MSG56*/"The 2D vector in ANG(v) function is zero length",
   /*MSG57*/"The 2D points in ANG(apex,p1,p2) function coincide",
   /*MSG58*/"The points 'p1' and 'p2' in function PLD(dist,p1,p2) coincide",
   /*MSG59*/"The three points defining the plane are coincident or collinear",
   /*MSG60*/"Function %s was already registered",                            /*60*/
   /*MSG61*/"The two 2D points defining line in %s function coincide",
   /*MSG62*/"The line lies on the plane",
   /*MSG63*/"Use CVUNIT(value,from_units,to_units)",
   /*MSG64*/"CVUNIT failed to convert units",
};


/****************************************************************************/
/*.doc error(external)*/
/*+
  Error reporting function. 'n' is the error number, 'str' is an additional 
  string argument which is printed only in some error messages. 

  If the error number 'n' is 0, then the string 'str' is printed 
  instead of a message from the 'error_messages' array.

  The function does nothing if 'cal_err' is already non-zero, avoiding 
  chained error messages.
-*/
/****************************************************************************/


void
/*FCN*/error(n, str)

  int  n;                             /* Error number                       */
  char *str;                          /* Additional string to print or NULL */
{
    cal_lex.sym = no_sym;

    if (cal_err != 0)
        return;

    if (n != 0) {
        cal_err = n;
    } else {
        cal_err = -999;
	if (str == NULL)
	    return;                   /* Just assign cal_err = -999 */
    }
    
    if (str == NULL) 
        str = "";

    ads_printf(/*MSG65*/"\n>> Error: ");
    if ((cal_err < 0) || (cal_err >= ELEMENTS(error_messages))) {
        ads_printf("%d\n\n", cal_err);
    } else {
        ads_printf(error_messages[cal_err], str);
        ads_printf(".\n\n");
    }
} /*error*/


