/* Next available MSG number is  63 */

/* PTEXT.C

      Copyright (C) 1990-1994 by Autodesk, Inc.
	 
      Permission to use, copy, modify, and distribute this software 
      for any purpose and without fee is hereby granted, provided 
      that the above copyright notice appears in all copies and that 
      both that copyright notice and this permission notice appear in 
      all supporting documentation.

      THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED
      WARRANTY.  ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR
      PURPOSE AND OF MERCHANTABILITY ARE HEREBY DISCLAIMED.



   Version 1.10

   This file contains support routines for the AutoLisp function PTEXT.lsp.
   Ptext.lsp was originally written entirely in AutoLisp, but it was found
   to run far too slowly to be useable, due to the overhead of doing string
   manipulation via Lisp.  Therefore this set of routines which handles the
   getting of characters from the keyboard, and all of the cursor controls
   was ported to C as an ADS application callable by the Lisp application.

   It is anticipated that this method of translating portions of AutoLisp
   code to C will satisfy many of the needs for applications which operate
   at acceptable speeds.  This is an example of how data can be passed from
   AutoLISP to ADS for evaluation.

   Most of the functions in this file have direct counterparts on the
   AutoLISP side, though it was not always possible or desireable to make
   a direct translation of the AutoLISP code in C.  This may account for
   inefficiencies in programming style.

   Version 1.00b contains a fix for the backspace of long lines after ^B
   problem reported by Keith Funk, 05 Dec 1990.

   Version 1.00c replaces the ads_entmake() text entity length method with
   an ads_entmod() method.  The problem was again reported by Keith Funk,
   Dec 1990.

   Version 1.10 fixes a number of coding problems turned up by a code review.

 */


#include  <stdio.h>
#include  <ctype.h>
#include  <string.h>
#include  "adslib.h"

/* Some handy definitions and macros */

#ifndef M_PI
#define M_PI	3.14159265358979323846
#endif

#define copy_pt(a, b) a[X]=b[X]; a[Y]=b[Y]; a[Z]=b[Z]; /* copy b to a */
#define round(f) (int)(f + 0.5);

#define GOOD 1
#define BAD 0

#define MAXFONTNAMESIZE 32

#ifndef DEBUG
#define NDEBUG 1
#endif	/* DEBUG */

/* ADSSERT.H -- Special assertion handling for ADS applications. */

#ifndef NDEBUG
#define assert(ex)      \
    {if (!(ex)){ads_printf(/*MSG1*/"\n\
Assertion (%s) failed: file \"%s\", line %d\n",/*MSG0*/" ex ",  \
    __FILE__,__LINE__); \
    ads_abort(/*MSG2*/"Assertion failed.");}}
#else
#define assert(ex)
#endif

enum direction { RIGHT, LEFT, UP, DOWN, TOP, BOTTOM, DELETE, HOME, END };

/*  General purpose queue, circular, bidirectional  */

struct queue {
    struct queue *qinext,	      /* Next item in queue */
		 *qiprev;	      /* Previous item in queue */
};

/* This structure contains the current text parameters at any time. */

struct strparam {
    ads_point pl;		      /* Left point */
    ads_point pr;		      /* Right point */
    int jt;			      /* Justification type */
    ads_real th;		      /* Text height */
    ads_real wf;		      /* Width factor */
    ads_real ra;		      /* Rotation angle */
    ads_real oa;		      /* Obliquing angle */
    char fn[MAXFONTNAMESIZE];	      /* Font name */
};

/* Forward functions */

void	main		_((int, char **));
void	dofun		_((void));
int	arg_err1	_((int, int, int));
int	arg_err2	_((int, int));
int	pt_itoa 	_((int, char *));
int	pt_GEX		_((int));
void	qinit		_((struct queue *));
void	qinsert 	_((struct queue *, struct queue *));
void	qappend 	_((struct queue *, struct queue *));
void	qremove 	_((struct queue *));
struct queue *qnext	_((struct queue *, struct queue *));
struct queue *qprev	_((struct queue *, struct queue *));
struct queue *qlast	_((struct queue *));
int	get_ll_item_number _((char *));
int	get_enames_into_list _((FILE *));
struct e_handles *find_entity_handle _((char *));
void	pr_ehand	_((void));
int	set_up		_((void));
int	process_line	_((char *));
int	pt_mod_ent	_((ads_name, int, struct resbuf *));
int	pt_getc 	_((void));
void	help_screen	_((void));
int	process_newline _((void));
ads_real get_line_length _((int));
int	measure_text	_((char *, ads_name));
struct resbuf *assoc_rb _((struct resbuf *, int));
int	set_text_params _((struct resbuf *));
int	create_text	_((struct strparam *, char *, ads_name, int));
char *	underline_string _((char *, int, int));
int	add_char_to_string _((int, int));
struct strloc parse_str_to_visual_chars _((char *, int));
struct strloc parse_str_to_real_chars _((char *, int));
int	move_cursor	_((enum direction, int));
int	delete_a_line	_((int));
int	pt_move_lines	_((enum direction, int));
int	wrap_at_word	_((void));
int	process_existing_entity _((void));
void	pt_strrev	_((char *, char *));
int	get_maximum_string_position _((void));
int	process_word_wrap _((void));
int	process_existing_entity _((void));

/* End forward functions */

int pt_mnl;
struct resbuf *ce, *cb;
struct resbuf *getargs = NULL, *rb = NULL;
ads_point pt_spt, pt_spt2, pt_rpt;
ads_real pt_ra, pt_mll, pt_ils, pt_ver;
ads_real pt_twf = 1.0, pt_th = 0.2;
ads_name pt_le;
char *pt_sty;

/* FUNCTION MAIN */

void
/*FCN*/main(argc,argv)
  int argc;
  char *argv[];
{
    short scode = RSRSLT;	      /* normal result code */
    int stat;
    char errmsg[80];

    ads_init(argc, argv);	      /* Initiation of communication
					 with AutoLisp begins here */
    for ( ;; ) {		      /* Request/Result loop. */
	if ((stat = ads_link(scode)) < 0) {
            sprintf(errmsg, /*MSG3*/"ptext: bad status from ads_link() = %d\n", stat);
#ifdef Macintosh
	    macalert(errmsg);
#else
            printf("%s",errmsg);
	    fflush(stdout);
#endif /* Macintosh */
	    exit(1);
	}

	scode = RSRSLT; 	      /* default return code */

	switch (stat) {

	case RQXLOAD:		      /* Load & register function */
            if (!ads_defun(/*MSG0*/"pt_GEX", 0))
		return;
	    continue;

	case RQXUNLD:		      /* Unloading */
	    break;

	case RQSUBR:		      /* AutoLisp has requested */
	    dofun();		      /* an externally defined function */

	    /* if (ce) ads_relrb(ce); */
	    ce = NULL;
	    ads_retvoid();
	    break;

	default:
	    break;
	}
    }
}

struct strparam *spm = NULL;

ads_real pt_wf = 1.0;
int modent;

/* Initialize static strings. */
typedef enum {
	  POINT, REAL, INTEGER, ARG, FEW, MANY, TEXT, FUNCAN, STR, AERR
	 } statstrvocab;

struct {
	char *sstr;
	statstrvocab scode;
} sstrs[] = {
          {/*MSG4*/"Point", POINT},
          {/*MSG5*/"Real", REAL},
          {/*MSG6*/"Integer", INTEGER},
          {/*MSG7*/"Arg.", ARG},
          {/*MSG8*/"few", FEW},
          {/*MSG9*/"many", MANY},
          {/*MSG10*/"Text: ", TEXT},
          {/*MSG11*/"Function cancelled.", FUNCAN},
          {/*MSG12*/"String", STR},
          {/*MSG19*/"\nCouldn't allocate enough memory. ", AERR}
	 };

void
/*FCN*/dofun()
{
    extern int grp_72;
    extern struct strparam *spm;
    int edit_t, val = 0, err;
    char *deft;

    deft = /*MSG0*/"256 byte buffer for Mtext get_line_length \
function\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";

    /* Malloc space for text parameters structure. */
    spm = (struct strparam *)malloc((unsigned)sizeof(struct strparam));
    if (!spm) {
	ads_fail(sstrs[(int)AERR].sstr);
	return;
    }
    spm->pl[X] = 0.0;
    spm->pl[Y] = 0.0;
    spm->pl[Z] = 0.0;
    spm->pr[X] = 0.0;
    spm->pr[Y] = 0.0;
    spm->pr[Z] = 0.0;
    spm->jt = 0;
    spm->th = 0.2;
    spm->wf = 1.0;
    spm->ra = 0.0;
    spm->oa = 0.0;

    /* Get the required arguments */
    if ((getargs = ads_getargs()) == NULL)
	return;

    /* Check the type and number of args */
    rb = getargs;

    /* Argument 1 -- ads_point pt_spt; -- Text start point */

    if (arg_err1(RT3DPOINT, 1, POINT) == BAD)
	return;
    copy_pt(pt_spt, rb->resval.rpoint);
    copy_pt(spm->pl, pt_spt);
    if (arg_err2(2,FEW) == BAD)
	return;

    /* Argument 2 -- ads_point pt_rpt; -- Text end point */

    if (arg_err1(RT3DPOINT, 2, POINT) == BAD)
	return;
    copy_pt(pt_rpt, rb->resval.rpoint);
    copy_pt(spm->pr, pt_rpt);
    if (arg_err2(3,FEW) == BAD)
	return;

    /* Argument 3 -- int grp_72; -- Justification type */

    if (arg_err1(RTSHORT, 3, INTEGER) == BAD)
	return;
    spm->jt = grp_72 = rb->resval.rint;
    if (arg_err2(4,FEW) == BAD)
	return;

    /* Argument 4 -- ads_real pt_th; -- Text height */

    if (arg_err1(RTREAL, 4, REAL) == BAD)
	return;
    spm->th = pt_th = rb->resval.rreal;
    if (arg_err2(5,FEW) == BAD)
	return;

    /* Argument 5 -- ads_real pt_ra; -- Rotation angle */

    if (arg_err1(RTREAL, 5, REAL) == BAD)
	return;
    spm->ra = pt_ra = rb->resval.rreal;
    if (arg_err2(6,FEW) == BAD)
	return;

    /* Argument 6 -- ads_real pt_mll; -- Maximum line length */

    if (arg_err1(RTREAL, 6, REAL) == BAD)
	return;
    pt_mll = rb->resval.rreal;
    if (arg_err2(7,FEW) == BAD)
	return;

    /* Argument 7 -- ads_real pt_ils; -- Inter-line spacing */

    if (arg_err1(RTREAL, 7, REAL) == BAD)
	return;
    pt_ils = rb->resval.rreal;
    if (arg_err2(8,FEW) == BAD)
	return;

    /* Argument 8 -- ads_real pt_ver; -- Version number */

    if (arg_err1(RTREAL, 8, REAL) == BAD)
	return;
    pt_ver = rb->resval.rreal;
    if (arg_err2(9,FEW) == BAD)
	return;

    /* Argument 9 -- char *pt_sty; -- Current style name */

    if (arg_err1(RTSTR, 9, STR) == BAD)
	return;
    pt_sty = rb->resval.rstring;
    strcpy(spm->fn, pt_sty);
    if (arg_err2(10,FEW) == BAD)
	return;

    /* Argument 10 -- int edit_t; -- Editing existing text flag */

    if (arg_err1(RTSHORT, 10, INTEGER) == BAD)
	return;
    edit_t = rb->resval.rint;
    if (arg_err2(11,MANY) == BAD)
	return;

    if ((create_text(spm, deft, pt_le, TRUE)) != RTNORM) {
	return;
    }  else {
	modent = TRUE;
    }


    if ((val = ads_getfuncode()) < 0)
	return;

    switch (val) {

    case 0:
	err = pt_GEX(edit_t);
	switch (err) {
	case RTFAIL:
	    exit(-1);
	    break;
	default:
	    break;
	}
	break;
    default:
	break;
    }
}

/*-----------------------------------------------------------------------*/

int
/*FCN*/arg_err1(arg_type, arg_num, arg_str)
  int arg_type;
  int arg_num;
  int arg_str;
{
    if (rb->restype != arg_type) {
        ads_printf(/*MSG13*/"\nPTEXT called with type %d. ", rb->restype);
        ads_printf(/*MSG14*/"\nERROR:  %s %d --  %s expected. ",
		   sstrs[(int)ARG].sstr,
		   arg_num, sstrs[arg_str].sstr);
	return BAD;
    }
    return GOOD;
}

int
/*FCN*/arg_err2(arg_num, arg_str)
  int arg_num;
  int arg_str;
{
    rb = rb->rbnext;		      /* Sets rb to NULL after last arg. */

    if (arg_str == (int)MANY) {
	if (rb != NULL) {
            ads_printf(/*MSG15*/"\nERROR:  %s %d --  Too %s arguments. ",
		       sstrs[(int)ARG].sstr,
		       arg_num, sstrs[arg_str].sstr);
	    return BAD;
	}
    } else if (arg_str == (int)FEW) {
	if (rb == NULL) {
            ads_printf(/*MSG16*/"\nERROR:  %s %d --  Too %s arguments. ",
		       sstrs[(int)ARG].sstr,
		       arg_num, sstrs[arg_str].sstr);
	    return BAD;
	}
    }
    return GOOD;
}

/*-----------------------------------------------------------------------*/

/* These are the machine codes for the various cursor motion controls
   for both UNIX and DOS which are reported by calls to (ads_grread).

   Note that UNIX machines can redefine the low level codes and the
   actions of the keys marked "DELETE" and the backspace key, as well
   others.  They may not operate as documented.  Check the codes returned
   by (grread) "key press" against the table below to determine which
   codes to change for your system.

   They may need translation for other country keyboard codes.
 */
#define TRUE	      1
#define FALSE	      0
#define nil	      0
#define P_APPN	      1 	      /* ^A Append a space at current loc.rsp.*/
#define P_BEGL	      2 	      /* ^B Beginning of line.*/
#define P_DWNL	      4 	      /* ^D Down a line.*/
#define P_ENDL	      5 	      /* ^E End of line.*/
#define P_BACK	      8 	      /* ^H Backspace.*/
#define P_ISRT	      9 	      /* ^I Toggle insert/overwrite mode.*/
#define P_LEFT	     12 	      /* ^L Left  -- non-destructive cursor.*/
#define P_RTRN	     13 	      /*    Return.*/
#define P_ENDT	     14 	      /* ^N End of text.*/
#define P_RGHT	     18 	      /* ^R Right -- non-destructive cursor.*/
#define P_BEGT	     20 	      /* ^T Beginning of text.*/
#define P_UPLD	     21 	      /* ^U Up a line (DOS).*/
#define P_UPLU	     24 	      /* ^U Up a line (UNIX).*/
#define P_QUIT	     26 	      /* ^Z Exit text entry.*/
#define P_SPCE	     32 	      /*    Spacebar.*/
#define P_PRCT	     37 	      /* %  Percent character.*/
#define P_HYPH	     45 	      /* -  Hyphen character.*/
#define P_UDEL	    127 	      /*    Delete character (UNIX).*/
#define P_DDEL	    211 	      /*    Delete character (DOS).*/
#define P_HLPD	    222 	      /*^F1 Help screen on DOS*/
#define P_HLPU	     31 	      /*^?  Help screen on UNIX*/

/* The following structure is used to store the string position information,
   which can be much different than it would seem at first.  Consider the
   following string:  "T%%uh%%oi%%os%%u is a test."  This string will
   display on the AutoCAD graphics screen as:

	 _
       This is a test.
	___


   showing a total of 15 visual characters including the period.  The
   actual string contains 12 more than that or 27 characters.  So if the
   cursor was positioned at the "i" in "This", the structure loc should
   contain the values loc.rsp == 9, loc.vsp == 3, and loc.nch == 1.

   Loc.nch would be 3 for "%%d", "%%p", etc., and 5 if the character were
   something like "%%127".  The real string position should always point
   to the final character in the visual string position.
 */

struct strloc {
    int rsp;			      /* Real String Position	*/
    int vsp;			      /* Visual String Position */
    int nch;			      /* # of Characters at vsp */
} loc;

/* This structure is similar to the general result buffer in ADS.
   It is used to store the list of "handles" of the text entities
   while processing them.
 */
struct e_handles {
    struct queue dllist;
    char *hand; 		      /* The handle */
} hand_list;

struct e_handles *eh = NULL, *ehhead = NULL;
struct strloc loc, tloc;
ads_name pt_te, pt_temp;
ads_real line_l;

int pnding, insert, pt_msp, pt_cl, grp_72;
int pt_ol = 0, pt_ul = 0, err;
char *str, *tstr, *s1;
char s[256], t[256];


/*  PT_ITOA -- Integer code to ASCII translation, base 10 assumed.
    This is provided because different platforms support this
    with different numbers of arguments.

    The length of the string is returned -- should be 1.
 */
int
/*FCN*/pt_itoa(i, cp)
  int i;
  char *cp;
{
    sprintf(cp, "%c", i);
    return strlen(cp);
}

/*
   pt_GEX -- This is the main processing loop called by the Lisp module.
   It handles all of the text entry and editing functions.
 */
int
/*FCN*/pt_GEX(edit_t)
  int edit_t;			      /* Editing existing text */
{

    FILE *fd;

    tstr = t;
    str = s;
    insert = TRUE;
    pt_cl = pt_mnl = 0;
    line_l = 0.0;
    pnding = nil;

    /* Initialize all structure members to NULL for portability. */

    hand_list.dllist.qinext = NULL;
    hand_list.dllist.qiprev = NULL;
    hand_list.hand = NULL;

    ehhead = NULL;
    ehhead = (struct e_handles *)malloc((unsigned)sizeof(hand_list));
    if (!ehhead) {
	ads_fail(sstrs[(int)AERR].sstr);
	return 0;
    }
    ehhead->hand = NULL;
    ehhead->dllist.qinext = NULL;
    ehhead->dllist.qiprev = NULL;


    /* Initilize the doubly-linked circular list */
    qinit(&ehhead->dllist);

    /* edit_t is non-zero if the user has selected text for editing.
       Therefore, open the handle file and put them into a doubly
       linked list.
     */

    if (edit_t) {

        if ((fd = fopen(/*MSG0*/"ptext.hdl", /*MSG0*/"r+")) == NULL) {
            ads_printf(/*MSG17*/"\n\
Couldn't open \"ptext.hdl\" for reading/writing. ");
	    return RTERROR;
	}

	/* Set the max. no. of lines. */
	pt_mnl = get_enames_into_list(fd);

	eh = ehhead;		      /* Set pointer to head of list */

	/* Get the entity name */
	if ((err = ads_handent(eh->hand, pt_te)) != RTNORM)
	    return err;

	/* Retrieve entity data */
	if ((ce = ads_entget(pt_te)) == NULL)
	    return RTERROR;
	cb = assoc_rb(ce, 1);	      /* Retrieve entity string */
	strncpy(str, cb->resval.rstring, 255); /* Copy string from resbuf */
	cb = assoc_rb(ce, 72);	      /* Retrieve just. type */
	grp_72 = cb->resval.rint;     /* Assign to variable */

    } else {
	eh = (struct e_handles *)malloc((unsigned)sizeof(hand_list));
	if (!eh) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}
	eh->hand = NULL;
	eh->dllist.qinext = NULL;
	eh->dllist.qiprev = NULL;

	/* pt_mnl set later, after first entity acquired. */

	eh = ehhead;		      /* Set pointer to head of list */

	str = NULL;
    }
    /* Wierd, but this causes the editor to act like the DTEXT command
       while entering text and the cursor is at the end of all text
       being entered.  The variable "str" is equal to NULL upon initial
       entry to the routine;  thereafter it will be either the string
       " " (a single space) if the user presses RETURN at the end of a
       line of text, or it will contain the string of characters remaining
       after the line is word-wrapped, or the characters following the
       cursor if RETURN is pressed within a line of characters.

       If (str == ""), i.e., a null string, then drop out of the loop.
       else continue on...
     */
    while ((str == NULL) || (strlen(str) > 0)) {
	if ((err = set_up()) != RTNORM)       /* Set up */
	    return err;
	while ((err = pt_getc()) == TRUE) {  /* Get characters */

            /* Maximum line length plus "slack" amount not yet reached... */

	    if (line_l < pt_mll) {
		/* Process the line */
		if ((err = process_line(tstr)) != RTNORM)
		    return err;
	    } else {
		/* Set up to wrap at word */
		wrap_at_word();

                /* Do the wordy 'rappinghood */
		if ((err = process_word_wrap()) != RTNORM)
		    return err;
	    }
	}
    }
    ads_printf(/*MSG18*/"\nExiting ptext.");
    return RTNORM;
}

/*  QINIT  --  Initialise links for null queue */

void
/*FCN*/qinit(qhead)
  struct queue *qhead;
{
    qhead->qinext = qhead->qiprev = qhead;
}

/*  QINSERT  --  Insert `object' in front of `qhead'. */

void
/*FCN*/qinsert(qhead, object)
  struct queue *qhead, *object;
{
    object->qinext = qhead;
    object->qiprev = qhead->qiprev;
    qhead->qiprev = object;
    object->qiprev->qinext = object;
}

/*  QAPPEND  --  Insert `object' after `qhead'. */

void
/*FCN*/qappend(qhead, object)
  struct queue *qhead, *object;
{
    object->qiprev = qhead;
    object->qinext = qhead->qinext;
    qhead->qinext = object;
    object->qinext->qiprev = object;
}

/*  QREMOVE -- Remove `object' from queue. */

void
/*FCN*/qremove(object)
  struct queue *object;
{
    object->qiprev->qinext = object->qinext;
    object->qinext->qiprev = object->qiprev;
}

/*  QNEXT  --  Get next object from queue nondestructively.
	       Returns NULL if already at end of queue.
 */

struct queue *
/*FCN*/qnext(qthis, qhead)
  struct queue *qthis;
  struct queue *qhead;
{
    struct queue *object;

    if ((object = qthis->qinext) == qhead) {
	object = NULL;
    }
    return object;
}

/*  QPREV  --  Get previous object from queue nondestructively.
	       Returns NULL if already at top of queue.
 */

struct queue *
/*FCN*/qprev(qthis, qhead)
  struct queue *qthis;
  struct queue *qhead;
{
    struct queue *object;

    if ((object = qthis->qiprev) == qhead->qiprev)
	object = NULL;
    return object;
}

/*  QLAST  --  Get the last object from queue nondestructively. */

struct queue *
/*FCN*/qlast(qhead)
  struct queue *qhead;
{
    struct queue *object;

    object = qhead;

    while (object->qinext != qhead)
	object = object->qinext;

    return object;
}

/*  GET_LL_ITEM_NUMBER --

    Calculate the position of the given item in the linked list
    from the top of the list.  Equates to the line number of the
    text entity.  Return the line number.
 */
int
/*FCN*/get_ll_item_number(hand)
  char *hand;
{
    int i = 1;
    struct e_handles *eh;

    eh = ehhead;

    while ((eh != NULL) && (strcmp(hand, eh->hand) != 0)) {
	eh = (struct e_handles *)qnext(&eh->dllist,&ehhead->dllist);
	i++;
    }
    return i;
}

/*  GET_ENAMES_INTO_LIST -- Read the file "fd" and place the handles
    found in it into a doubly linked list of entity handles.
 */
int
/*FCN*/get_enames_into_list(fd)
  FILE *fd;
{
    struct e_handles *ehtemp = NULL;
    int sl, count = 0;
    char *s1, s[256];

    s1 = s;

    while (!feof(fd)) {
	eh = (struct e_handles *)malloc((unsigned)sizeof(hand_list));
	if (!eh) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}
	eh->hand = NULL;
	eh->dllist.qinext = NULL;
	eh->dllist.qiprev = NULL;

	count++;
	if (count == 1) {
	    ehtemp = eh = ehhead;
	}

	fgets(s1, 256, fd);
        strtok(s1, "\n");
	sl = strlen(s1);

	eh->hand = (char *)malloc((unsigned)(sl+1)*(sizeof(char)));
	if (!eh->hand) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}

	strcpy(eh->hand, s1);

	/* Add the entity handle to the linked list. */
	qappend(&ehtemp->dllist, &eh->dllist);
	ehtemp = eh;
    }
    fclose(fd);
    return count;
}

/* FIND_ENTITY_HANDLE -- Find the element associated with "hand". */

struct e_handles *
/*FCN*/find_entity_handle(hand)
  char *hand;
{
    struct e_handles *j;

    j = ehhead;
    while ((j) && (j->hand != NULL)) {
	if (strcmp(hand, j->hand) == 0)
	    return j;
	j = (struct e_handles *)qnext(&j->dllist, &ehhead->dllist);
    }
    ads_printf(/*MSG20*/"\nHandle %s not found. ", hand);
    return NULL;
}

/* PR_EHAND -- Print all of the elements in order.
   Not used.  Use for debugging queues.
 */
void
/*FCN*/pr_ehand()
{
    struct e_handles *j;

    j = ehhead;
    while (j) {
        ads_printf(/*MSG21*/"\n\
Item address: %x, next: %x, prev: %x, handle: %s",
		   &j->dllist, j->dllist.qinext, j->dllist.qiprev, j->hand);
	j = (struct e_handles *)qnext(&j->dllist, &ehhead->dllist);
    }
}


/* SET_UP

   Set up before getting keyboard input.

   The counter "pt_cl" is the number of the current line
   starting at "1".
 */
int
/*FCN*/set_up()
{

    pt_cl++;			      /* check against get_ll_item_number(); */

    if (str == NULL) {
	str = s;
        strcpy(str, "");
    }

    if (strlen(str) == 0) {
        strcpy(str, " ");
	loc.rsp = 1;
	loc.vsp = 1;
	loc.nch = 1;
	pt_msp	= 1;
    } else {

	loc = parse_str_to_visual_chars(str, 1);

	/* set maximum string position (pt_msp) after allowing for */
	/* various control character codes such as %%d or %%p.	  */
	tloc = parse_str_to_real_chars(str, strlen(str));

	pt_msp = tloc.vsp;

	/* Check for overrun on string length */
	assert(pt_msp < 255);
    }

    /* set string that gets printed on-screen via (ads_entmod).    */
    tstr = underline_string(str, loc.rsp, insert);


    /* Process the line */
    if (process_line(tstr) != RTNORM)
	return 0;

    ads_printf("\n%s%s", sstrs[(int)TEXT].sstr, str);    /* "Text: " */
    return RTNORM;
}

/*  PROCESS_LINE */
int
/*FCN*/process_line(tstr)
  char *tstr;
{

    struct resbuf rb1, *rb;
    struct e_handles *ehnew;
    struct e_handles *ehtemp;
    int sl;

    /* Initialize all structure members to NULL for portability. */
    rb = &rb1;
    ehnew = (struct e_handles *)malloc((unsigned)sizeof(hand_list));
    if (!ehnew) {
	ads_fail(sstrs[(int)AERR].sstr);
	return 0;
    }
    ehnew->hand = NULL;
    ehnew->dllist.qinext = NULL;
    ehnew->dllist.qiprev = NULL;

    if (ce) {			      /* Modify the text string */
	rb->restype = 1;
	rb->resval.rstring = tstr;
	set_text_params(ce);
	/* Modify the current entity to place the cursor. */
	if (pt_mod_ent(pt_te, rb->restype, rb) != RTNORM)
	    return 0;
    } else {			      /* Draw the text string */

	if (create_text(spm, tstr, pt_te, FALSE) != RTNORM)
	    return 0;

	/* Retrieve entity data */
	if ((ce = ads_entget(pt_te)) == NULL)
	    return RTERROR;

	cb = assoc_rb(ce, 5);	      /* Retrieve entity handle */
	sl = strlen(str);

	/* Malloc space for the handle */
	ehnew->hand = (char *)malloc((unsigned)(strlen(cb->resval.rstring)+1) *
				     sizeof(char));
	if (!ehnew->hand) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}

	/* Assign the handle from the resbuf */
	strcpy(ehnew->hand, cb->resval.rstring);

	/* Add the entity handle to the linked list. */
	if (ehhead->hand == NULL) {
	    /* Malloc space for the handle */
	    ehhead->hand = (char *) malloc((unsigned)sl+1);
	    if (!ehhead->hand) {
		ads_fail(sstrs[(int)AERR].sstr);
		return 0;
	    }
	    strcpy(ehhead->hand, ehnew->hand);
	    eh = ehhead;
	    pt_mnl = 1;
	} else if (eh == NULL) {
	    qappend(&ehhead->dllist, &ehnew->dllist);
	    pt_mnl++;
	} else {
	    qappend(&eh->dllist, &ehnew->dllist);
	    pt_mnl++;
	}

	assert(pt_cl == get_ll_item_number(cb->resval.rstring));

	ehtemp = eh;
	if (qnext(&ehtemp->dllist,&ehhead->dllist) != NULL)
	    eh = (struct e_handles *)qnext(&ehtemp->dllist,&ehhead->dllist);


    }
    return RTNORM;
}

/*  PT_MOD_ENT -- Modify the entity indicated by "en". */
int
/*FCN*/pt_mod_ent(en, rbtype, rbnew)
  ads_name en;			      /* Entity name */
  int rbtype;			      /* Type of value -- DXF code */
  struct resbuf *rbnew; 	      /* Resbuf containing new value */
{
    struct resbuf *ce, *cb;
    char *temp;
    int sl;

    temp = tstr;
    sl = strlen(temp);

    /* Retrieve entity data */
    if ((ce = ads_entget(en)) == NULL)
	return RTERROR;

    cb = assoc_rb(ce, rbtype);
    if (((rbtype > -1) && (rbtype < 10)) || (rbtype == 999)) {
	if (strlen(cb->resval.rstring) < sl) {
	    cb->resval.rstring = (char *)realloc(cb->resval.rstring,
						 (unsigned) sl+1);
	}
	if (!cb->resval.rstring) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}
	cb->resval.rstring = rbnew->resval.rstring;
    } else if ((rbtype == 10) ||
	       (rbtype == 11) ||
	       (rbtype == 12) ||
	       (rbtype == 13) ||
	       (rbtype == 210)) {     /* Points */
	copy_pt(cb->resval.rpoint, rbnew->resval.rpoint);
    } else if ((rbtype > 9) && (rbtype < 60)) {  /* Real numbers */
	cb->resval.rreal = rbnew->resval.rreal;
    } else if ((rbtype > 59) && (rbtype < 80)) { /* Integers */
	cb->resval.rint = rbnew->resval.rint;
    } else {
	/* Only regular entity data is checked;  Not extended */
        /* entity data.  Therefore I don't know what to do... */
	return 0;
    }

    if (ads_entmod(ce) != RTNORM) {
	return 0;
    }
    return RTNORM;
}

/*  PT_GETC -- This function gets characters from the keyboard while
    AutoCAD is running.  Only keyboard input is accepted.
 */
int
/*FCN*/pt_getc()
{
    int type, my_char = 0;
    int return_val = FALSE;

    char  *ans, a[17];
    struct resbuf cbuf, cmd;
    struct resbuf *ce, *cb;
    struct strloc tmp;
    int sl, err = 0;

    ans  = a;
    s1 = str;
    tstr = t;
    type = 0;


    sl = strlen(str);

    cmd.restype = RTSHORT;
    cmd.resval.rint = 0;
    ads_setvar(/*MSG0*/"cmdecho", &cmd);

    for (;;) {

	if ((err = ads_grread(0, &type, &cbuf)) != RTNORM) {

	    /* Clean up if cancelling... */
	    if (err < -5000) {
		/* Retrieve entity data */
		if ((ce = ads_entget(pt_te)) == NULL)
		    return RTERROR;

		cb = assoc_rb(ce, 1);
		sl = strlen(str);
		if (strlen(cb->resval.rstring) < sl) {
		    cb->resval.rstring = (char *)
					 realloc(cb->resval.rstring,
						 (unsigned) sl+1);
		}
		if (!cb->resval.rstring) {
		    ads_fail(sstrs[(int)AERR].sstr);
		    return 0;
		}
		cb->resval.rstring = str;
		if (ads_entmod(ce) != RTNORM) {
		    ads_relrb(ce);
		    return 0;
		}
                str = "";
		return FALSE;
	    } else {
                ads_fail(/*MSG22*/"Not valid data.");
                str = "";
		return FALSE;
	    }
	}



	/* Disallow all input except keyboard. */
	/* if (cbuf.restype == RTSHORT)  */
	if (type == 2) {
	    if (cbuf.restype == RTSHORT) {

		my_char = cbuf.resval.rint;

		switch(my_char) {

		case 3: 	      /* Control-C */
		    ads_fail(sstrs[(int)FUNCAN].sstr);
		    break;
		case P_APPN:	      /* Append to EOL */

		    add_char_to_string(my_char, TRUE);
		    loc = parse_str_to_visual_chars(str, loc.vsp);
		    return_val = TRUE;
		    break;
		case P_BEGL:	      /* ^B - Beginning of line. */
		    move_cursor(HOME, FALSE);	/* Move cursor */
		    return_val = TRUE;
		    break;
		case P_DWNL:	      /* ^D - Down a line. */
		    move_cursor(DOWN, FALSE);	/* Move cursor */
		    return_val = TRUE;
		    break;
		case P_ENDL:	      /* ^E - End of line. */
		    move_cursor(END, FALSE);	/* Move cursor */
		    return_val = TRUE;
		    break;
		case P_BACK:	      /* Backspace */
		    move_cursor(LEFT, TRUE);
		    return_val = TRUE;
		    break;
		case P_ISRT:	      /* ^I - Toggle insert/overwrite mode.*/
		    if (insert) {     /* Toggle insert mode...*/
			insert = FALSE;
                        ads_grtext(-2, /*MSG23*/"Ptext: Overwrite mode", 0);
		    } else {
			insert = TRUE;
                        ads_grtext(-2, /*MSG24*/"Ptext: Insert mode", 0);
		    }
		    return_val = TRUE;
		    break;
		case P_LEFT:	      /* ^L - Left. */
		    move_cursor(LEFT, FALSE);
		    return_val = TRUE;
		    break;
		case P_RTRN:	      /* Return Key */
		    if (process_newline() != RTNORM)
			return 0;
		    return_val = FALSE;
		    break;
		case P_ENDT:	      /* ^N - End of text. */
		    move_cursor(BOTTOM, FALSE);
		    return_val = TRUE;
		    break;
		case P_RGHT:	      /* ^R - Right. */
		    move_cursor(RIGHT, FALSE);
		    return_val = TRUE;
		    break;
		case P_BEGT:	      /* ^T - Top of text.. */
		    move_cursor(TOP, FALSE);
		    return_val = TRUE;
		    break;
		case P_UPLD:	      /* ^U - Up a line (DOS) */
		case P_UPLU:	      /* ^U - Up a line (UNIX) */
		    move_cursor(UP, FALSE);
		    return_val = TRUE;
		    break;
		case P_QUIT:	      /* ^Z - Quit. */
                    ads_initget(nil, /*MSG25*/"Yes No");
                    ads_getkword(/*MSG26*/"\nExit from text entry? <Y>: ",
				 ans);
                    if (strcmp(ans, /*MSG27*/"No") == 0) {
			return_val = TRUE;
                        ads_printf("\n%s%s", sstrs[(int)TEXT].sstr, str);
		    } else {
			/* Retrieve entity data */
			if ((ce = ads_entget(pt_te)) == NULL)
			    return RTERROR;

			cb = assoc_rb(ce, 1);
			sl = strlen(str);
			if (strlen(cb->resval.rstring) < sl) {
			    cb->resval.rstring = (char *)
						 realloc(cb->resval.rstring,
							 (unsigned) sl+1);
			}
			if (!cb->resval.rstring) {
			    ads_fail(sstrs[(int)AERR].sstr);
			    return 0;
			}
			cb->resval.rstring = str;
			if (ads_entmod(ce) != RTNORM) {
			    ads_relrb(ce);
			    return 0;
			}
                        str = "";
			return_val = FALSE;
		    }
		    break;
		case P_UDEL:	      /* Delete character (UNIX).*/
		case P_DDEL:	      /* Delete character (DOS).*/
		    move_cursor(DELETE, FALSE);
		    return_val = TRUE;
		    break;
		case P_HLPD:	      /*^F1 Help screen on DOS*/
		case P_HLPU:	      /*^?  Help screen on UNIX*/
		    help_screen();
		    return_val = TRUE;
		    break;
                case P_PRCT:          /* "%" */
		    switch(pnding) {
		    case nil:
			pnding = 1;
			break;
		    case 1:
			pnding = 2;
			break;
		    case 2:
			pnding = nil;
			break;
		    default:
			ads_exit(-2);
			break;
		    }
		    add_char_to_string(my_char, nil);
		    return_val = TRUE;
		    break;
		default:	      /* Regular Keyboard Item. */
		    if (pnding > 1) {
			switch(my_char) {
			case 111:     /* overline */
			case 117:     /* underline */
			case 100:     /* degrees */
			case 112:     /* plus/minus */
			case  99:     /* circle diameter symbol */
			case  32:     /* space */
			    pnding = -1;
			    add_char_to_string(my_char, nil);
			    break;
			case 48:      /* 0 */
			case 49:      /* 1 */
			case 50:      /* 2 */
			case 51:      /* 3 */
			case 52:      /* 4 */
			case 53:      /* 5 */
			case 54:      /* 6 */
			case 55:      /* 7 */
			case 56:      /* 8 */
			case 57:      /* 9 */
                            ads_printf("\n");
			    if (pnding < 4) {
				add_char_to_string(my_char, nil);
				pnding += 1;
			    } else {
				pnding = nil;
				add_char_to_string(my_char, nil);
			    }
			    break;
			default:
			    pnding = nil;
			    add_char_to_string(my_char, nil);
			    break;
			}
		    } else {
			add_char_to_string(my_char, nil);
		    }
		    return_val = TRUE;	  /* Return value */
		    break;
		}
	    }			      /* end -- if(cbuf.restype == RTSHORT) */
	    if (my_char != P_RTRN) {
		/* set current string postion (loc.vsp) after allowing for */
		/* various control character codes such as %%d or %%p.	   */
                /* But only if we've added a charater to the string.       */

		loc = parse_str_to_visual_chars(str, loc.vsp);

		if (sl != strlen(str)) {

		    /* set maximum string postion (pt_msp) after allowing for
		       various control character codes such as %%d or %%p.
		     */

		    tmp = parse_str_to_real_chars(str, strlen(str));
		    pt_msp = tmp.vsp;
		}


		/* set string that gets printed on-screen via (ads_entmod).*/

		tstr = underline_string(str, loc.rsp, insert);

		/* set line length up to current adjusted string position. */
		if (pnding == 0) {
		    line_l = get_line_length(loc.rsp);
		} else {
		    line_l = 0.0;
		}

		/* If Fit text... */
		if (grp_72 == 5) {
		    if ((line_l > pt_mll) &&
                        (my_char != ' ') &&
                        (my_char != '-')) {
			/* Keep accepting characters until */
			/* a space or hyphen is found...   */
			line_l = 0.0;
		    }
		}
	    }
	    return return_val;
	}
    }
}

/*-----------------------------------------------------------------------*/

/*  HELP_SCREEN -- Help screen for the text editor. */
void
/*FCN*/help_screen()
{
    struct resbuf cbuf;
    int type;

    ads_textpage();
    ads_printf("\n");
    ads_printf(/*MSG28*/"\n   \
The Autodesk Ptext editor, Version %4.2f", pt_ver);
    ads_printf("\n");
    ads_printf(/*MSG29*/"\n  \
^A  -- (A)ppend a space after the current cursor position");
    ads_printf(/*MSG30*/"\n         and move the cursor to that postition.");
    ads_printf(/*MSG31*/"\n  \
^B  -- Move the cursor to the (B)eginning of the line.");
    ads_printf(/*MSG32*/"\n  \
^D  -- Move the cursor (D)own a line; maintains the current");
    ads_printf(/*MSG33*/"\n         \
letter position.  This position may appear to be ");
    ads_printf(/*MSG34*/"\n         \
different due to character kerning within a font.");
    ads_printf(/*MSG35*/"\n  ^E  -- Go to the (E)nd of the current line.");
    ads_printf(/*MSG36*/"\n  ^H  -- Backspace key.");
    ads_printf(/*MSG37*/"\n  ^I  -- Toggle (I)nsert/overwrite mode.");
    ads_printf(/*MSG38*/"\n  ^L  -- Move the cursor to the (L)eft.");
    ads_printf(/*MSG39*/"\n  ^N  -- Move the cursor to the e(N)d of ");
    ads_printf(/*MSG40*/"\n         the last text entity in the list.");
    ads_printf(/*MSG41*/"\n  ^R  -- Move the cursor to the (R)ight.");
    ads_printf(/*MSG42*/"\n  ^T  -- Move the cursor to (T)op or the start of");
    ads_printf(/*MSG43*/"\n         the first text entity in the list.");
    ads_printf(/*MSG44*/"\n  ^U  -- Move the cursor (U)p a line.");
    ads_printf(/*MSG45*/"\n  ^Z  -- Exit text entry.");
    ads_printf("\n");
    ads_printf(/*MSG46*/"\nPress any key to return to your drawing.");
    ads_grread(0, &type, &cbuf);
    ads_graphscr();
    ads_printf("\r                                          ");
    ads_printf("\n");
    ads_printf("\n%s", sstrs[(int)TEXT].sstr);
    ads_printf(str);
}

/*  PROCESS_NEWLINE -- Process a newline character (Return). */
int
/*FCN*/process_newline()
{

    struct resbuf *rb, *eb;
    struct strloc tloc;

    char t2[256], *str2;
    int grp_72;

    str2 = t2;

    assert(ce != NULL);

    if (ce) {			      /* There should (!!!) always be
					 an entity at this point... */

	rb = assoc_rb(ce, 72);
	grp_72 = rb->resval.rint;

        /* Get correct "start point" for current type of text entity...  */
	/* This should correctly handle mixed text justification types.  */

	/* Right or Center justified */
	if ((grp_72 == 2) || (grp_72 == 1)) {

	    rb = assoc_rb(ce, 11);
	    copy_pt(pt_spt, rb->resval.rpoint);

	} else {

	    rb = assoc_rb(ce, 10);
	    copy_pt(pt_spt, rb->resval.rpoint);
	}
	ads_polar(pt_spt, pt_ra - (M_PI / 2), pt_ils, spm->pl);

	if (grp_72 == 5) {	      /* if Fit text */

	    rb = assoc_rb(ce, 11);
	    copy_pt(pt_rpt, rb->resval.rpoint);

	    ads_polar(pt_rpt, pt_ra - (M_PI / 2), pt_ils, spm->pr);

	}
	eb = assoc_rb(ce, 1);
	eb->resval.rstring = (char *)malloc((unsigned)(loc.rsp-loc.nch+1));
	if (!eb->resval.rstring) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}
	strncpy(eb->resval.rstring, str, loc.rsp-loc.nch);
        eb->resval.rstring[loc.rsp-loc.nch] = '\0';

        if (strcmp(eb->resval.rstring, "") == 0) {
	    eb->resval.rstring = (char *)malloc((unsigned)strlen(str)+1);
	    if (!eb->resval.rstring) {
		ads_fail(sstrs[(int)AERR].sstr);
		return 0;
	    }
            strcpy(eb->resval.rstring, " ");
	}
	/* Modify the current entity. */
	if (pt_mod_ent(pt_te, eb->restype, eb) != RTNORM)
	    return 0;


	strcpy(str2, str);

	if (pt_ol) {
            strcpy(tstr, "%%o");
	    if (pt_ul) {
                strcat(tstr, "%%u");
	    }
	    strcat(tstr, str + (loc.rsp-loc.nch));
	} else if (pt_ul) {
            strcpy(tstr, "%%u");
	    strcat(tstr, str + (loc.rsp-loc.nch));
	} else {
	    strcpy(tstr, str + (loc.rsp-loc.nch));
	}

	strcpy(str, tstr);
        s[strlen(tstr)] = '\0';

        /* Null line at end of paragraph, and previous line was " " ... */

	if ((qnext(&eh->dllist, &ehhead->dllist) == NULL) &&
            (strcmp(str," ") == 0) && (strcmp(str2," ") == 0)) {
            strcpy(str, "");          /* Exit from routine. */
	} else {
	    tloc = parse_str_to_visual_chars(str, 1);
	    tstr = underline_string(str, tloc.rsp, insert);

	    if (pt_move_lines(DOWN, FALSE) != RTNORM)
		return 0;
	    /* pt_mnl++; */
	}
	ce = NULL;
    } else {
        ads_fail(/*MSG47*/"\nError processing newline character. ");
    }
    return RTNORM;
}

/*  GET_LINE_LENGTH --

    Get the length of a text line by creating a dummy text entity
    on the frozen text layer.  This entity will contain the current
    text string without the underline/overline cursor characters
    up to the current cursor position.

    Return the distance between the right and left points of the
    right justified text string, then delete the entity.

 */
ads_real
/*FCN*/get_line_length(rsp)
  int rsp;
{
    struct resbuf *rb, *te;
    ads_name ename;
    ads_point pt, pt1;
    ads_real dist;
    char t[256];
    char *tmpstr;

    tmpstr = t;
    strncpy(tmpstr, str, rsp);
    t[rsp] = '\0';

    pt[0] = 0;
    pt[1] = 0;
    pt[2] = 0;

    if (modent) {
	ename[0] = pt_le[0];
	ename[1] = pt_le[1];

	/* Modify the temp entity. */
	if (measure_text(tmpstr, ename) != RTNORM)
	    return 0;
    } else {
	/* Create a right justified text entity with the current style */
	/* parameters at 0,0.  The entity name is returned.	       */
	if (create_text(spm, tmpstr, ename, TRUE) != RTNORM)
	    return 0;
    }

    /* Retrieve entity data */
    if ((te = ads_entget(ename)) == NULL)
	return RTERROR;

    /* Get the group 10 and 11 fields from the result buffer.	   */
    rb = assoc_rb(te, 10);
    copy_pt(pt, rb->resval.rpoint);

    rb = assoc_rb(te, 11);
    copy_pt(pt1, rb->resval.rpoint);

    dist = ads_distance(pt, pt1);

    if (!modent)
	if(ads_entdel(ename) != RTNORM)
	    return RTERROR;


    return dist;
}

/* MEASURE_TEXT -- Replace the measure text string on the frozen layer */
int
/*FCN*/measure_text(str, en)
  char *str;			      /* the string */
  ads_name en;			      /* the entity name */
{
    struct resbuf *te, *rb, *cb;
    int sl;

    /* Retrieve entity data */
    if ((te = ads_entget(en)) == NULL)
	return RTERROR;

    /* Get the group 1 (Text string) field from the result buffer.  */
    rb = assoc_rb(te, 1);

    sl = strlen(str);
    if (strlen(rb->resval.rstring) < sl)
	/* Alloc a new string buffer */
	rb->resval.rstring = (char *)realloc(rb->resval.rstring, sl+1);
    if (!rb->resval.rstring) {
	ads_fail(sstrs[(int)AERR].sstr);
	return 0;
    }
    strcpy(rb->resval.rstring, str);

    /* Get the group 40 (Text height) field from the result buffer.  */
    rb = assoc_rb(te, 40);
    cb = assoc_rb(ce, 40);
    if (rb->resval.rreal != cb->resval.rreal)
	rb->resval.rreal = cb->resval.rreal;

    /* Get the group 50 (Text rot) field from the result buffer.  */
    rb = assoc_rb(te, 50);
    cb = assoc_rb(ce, 50);
    if (rb->resval.rreal != cb->resval.rreal)
	rb->resval.rreal = cb->resval.rreal;

    /* Get the group 41 (Text width) field from the result buffer.  */
    rb = assoc_rb(te, 41);
    cb = assoc_rb(ce, 41);
    if (rb->resval.rreal != cb->resval.rreal)
	rb->resval.rreal = cb->resval.rreal;

    /* Get the group 51 (Text obliq.) field from the result buffer.  */
    rb = assoc_rb(te, 51);
    cb = assoc_rb(ce, 51);
    if (rb->resval.rreal != cb->resval.rreal)
	rb->resval.rreal = cb->resval.rreal;

    /* Get the group 7 (Text style) field from the result buffer.  */
    rb = assoc_rb(te, 7);
    cb = assoc_rb(ce, 7);
    sl = strlen(cb->resval.rstring);
    if (strcmp(rb->resval.rstring, cb->resval.rstring) != 0) {
	if (strlen(rb->resval.rstring) < sl)
	    rb->resval.rstring = (char *) realloc(rb->resval.rstring,
						  (unsigned) sl+1);
	if (!rb->resval.rstring) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}
	strcpy(rb->resval.rstring, cb->resval.rstring);
    }

    if (ads_entmod(te) != RTNORM) {
	ads_relrb(te);
	return 0;
    }
    return RTNORM;
}

#ifdef applec
#pragma segment /*MSG0*/'ptext2'
#endif

/*  ASSOC_RB -- Search a result buffer chain and return the first item
    associated with the specified group code.
 */
struct resbuf *
/*FCN*/assoc_rb(rb, grp)
  struct resbuf *rb;
  int grp;
{
    while ((rb != NULL) && (rb->restype != grp))
	rb = rb->rbnext;

    return rb;
}

/*  SET_TEXT_PARAMS -- Set some of the text parameters. */
int
/*FCN*/set_text_params(ce)
  struct resbuf *ce;
{
    struct resbuf *tb;

    /* Query and set text height */
    tb = assoc_rb(ce, 40);
    if (tb == NULL) {
	spm->th = pt_th;
    } else {
	spm->th = tb->resval.rreal;
    }

    /* Query and set width factor */
    tb = assoc_rb(ce, 41);
    if (tb == NULL) {
	spm->wf = pt_wf;
    } else {
	spm->wf = tb->resval.rreal;
    }

    /* Query and set rotation angle */
    tb = assoc_rb(ce, 50);
    if (tb == NULL) {
	spm->ra = pt_ra;
    } else {
	spm->ra = tb->resval.rreal;
    }

    /* Query and set obliquing angle */
    tb = assoc_rb(ce, 51);
    if (tb == NULL) {
	spm->oa = 0.0;
    } else {
	spm->oa = tb->resval.rreal;
    }

    /* Query and set style name */
    tb = assoc_rb(ce, 7);
    if ((tb == NULL) && (strcmp(spm->fn, "") == 0)) {
	strcpy(spm->fn, pt_sty);
    } else {
	strcpy(spm->fn, tb->resval.rstring);
    }

    return GOOD;
}

/*  CREATE_TEXT -- Create a text string entity on the frozen text
    layer if "flag" is set, else create it on the current layer.
 */
int
/*FCN*/create_text(pm, str, en, flag)
  struct strparam *pm;		      /* Text parameters */
  char *str;			      /* the string */
  ads_name en;			      /* the entity name */
  int flag;			      /* if 1, put text on frozen layer. */
{
    struct resbuf txh,		      /* header */
		  txl,		      /* layer */
		  lpt,		      /* insertion point */
		  rpt,		      /* insertion point */
		  sizeh,	      /* height of text */
		  strh, 	      /* string */
		  rota, 	      /* rotation angle */
		  wf,		      /* width factor */
		  obla, 	      /* obliquing angle */
		  jt,		      /* justification type */
		  fn;		      /* font name */

    struct resbuf *rb;
    struct resbuf gv;
    int i;

    if (ads_getvar(/*MSG0*/"TEXTSTYLE", &gv) != RTNORM)
	return RTERROR;
    rb = ads_tblsearch(/*MSG0*/"STYLE", gv.resval.rstring, 0);
    free(gv.resval.rstring);

    if (flag) {
	txh.rbnext = &txl;
	txl.rbnext = &lpt;
    } else {
	txh.rbnext = &lpt;
    }
    lpt.rbnext = &rpt;
    rpt.rbnext = &sizeh;
    sizeh.rbnext = &strh;
    strh.rbnext = &rota;
    rota.rbnext = &wf;
    wf.rbnext = &obla;
    obla.rbnext = &jt;
    jt.rbnext = &fn;
    fn.rbnext = NULL;

    txh.restype = 0;
    txh.resval.rstring = /*MSG0*/"TEXT";

    if (flag) {
	txl.restype = 8;
        txl.resval.rstring = /*MSG0*/"FROZEN_TEXT";
    }

    /* Center or right justified text */
    if ((pm->jt == 1)  || (pm->jt == 2)) {
	lpt.restype = 10;
	copy_pt(lpt.resval.rpoint, pm->pr);
	rpt.restype = 11;
	copy_pt(rpt.resval.rpoint, pm->pl);
    } else {
	lpt.restype = 10;
	copy_pt(lpt.resval.rpoint, pm->pl);
	rpt.restype = 11;
	copy_pt(rpt.resval.rpoint, pm->pr);
    }

    sizeh.restype = 40;
    if (pm->th != 0.0) {
	sizeh.resval.rreal = pm->th;
    } else {
	sizeh.resval.rreal = 0.2;
    }

    strh.restype = 1;
    strh.resval.rstring =  str;

    rota.restype = 50;
    rota.resval.rreal = pm->ra;

    wf.restype = 41;
    wf.resval.rreal = pm->wf;

    obla.restype = 51;
    obla.resval.rreal = pm->oa;

    jt.restype = 72;
    if (flag) {
	jt.resval.rint = 2;
    } else {
	jt.resval.rint = pm->jt;
    }

    fn.restype = 7;
    if (strlen(pm->fn) == 0) {
	if ((rb = assoc_rb(rb, 2)) == NULL)
	    return BAD;
	strcpy(pm->fn, rb->resval.rstring);
	fn.resval.rstring = rb->resval.rstring;
    } else {
	fn.resval.rstring = pm->fn;
    }

    if ((i = ads_entmake(&txh)) != RTNORM)
	return i;

    if ((i = ads_entlast(en)) != RTNORM)
	return i;
    return RTNORM;
}

/*  UNDERLINE_STRING -- Turn on underlining for a "Visual" character
    location (j) in a string (s).  Return the string with underlining.
 */
char *
/*FCN*/underline_string (s, j, insert)
  char *s;			      /* The string to under/overscore. */
  int j;			      /* The location of under/overscore. */
  int insert;			      /* if (insert) then underscore only. */
{
    int temp;
    char *s1, *ulstr;

    s1 = s;

    temp = strlen(s);
    ulstr = malloc((unsigned)temp+13);
    if (!ulstr) {
	ads_fail(sstrs[(int)AERR].sstr);
	return 0;
    }

    if ((temp > 0) && (strcmp(s, " ") != 0)) {
	if ((j > 0) && (j <= temp)) {
	    if (loc.rsp-loc.nch > 0) strncpy(ulstr, s, loc.rsp-loc.nch);
            ulstr[loc.rsp-loc.nch] = '\0';
            strcat(ulstr, "%%u");
            if (!insert) strcat(ulstr, "%%o");
	    s = s1+loc.rsp-loc.nch;
	    strncat(ulstr, s, loc.nch);
            if (!insert) strcat(ulstr, "%%o");
            strcat(ulstr, "%%u");
	    s = s1 + loc.rsp;
	    strcat(ulstr, s);
	} else {
	    strcpy(ulstr, s);
            strcat(ulstr, "%%u %%u");
	}
    } else {
        strcpy(ulstr, "%%u %%u");
    }
    s = s1;
    return ulstr;
}

/*  ADD_CHAR_TO_STRING -- Add a character (my_char) to a string (str).
    Set the new values in the structure `loc' and return loc.vsp,
    the new visual string position.
 */
int
/*FCN*/add_char_to_string(my_char, appnd)
  int my_char;
  int appnd;
{

    char *nstr, n[256];

    nstr = n;

    if (pt_msp < 255) {

	if (loc.vsp <= pt_msp) {
	    if (appnd == TRUE) {      /* If appending... */
		s1 = str+loc.rsp;
                strcpy(nstr, " ");    /* Start with a space... or, */
	    } else {
		s1 = str+(loc.rsp-loc.nch);
		/* Start temp string with new char */
		if (pt_itoa(my_char, nstr) == 0) ads_exit(-3);
	    }

	    /* Add the rest of the old string. */
	    if (insert == 0) {	      /* If overwriting... */
                if (strcmp(s1+loc.nch, "") == 0) {
		    pt_msp++;
                    strcat(nstr, " ");
		} else {
		    strcat(nstr, s1+loc.nch);
		}
	    } else {
		strcat(nstr, s1);
	    }

	    strcpy(s1, nstr);
	}
        ads_printf("\r%s%s", sstrs[(int)TEXT].sstr, str);

        /* `pnding' can be set to -1 just before a call to this function
	   when any of the following codes is found:
	     %%%, %%d, %%p, %%c, %%o, %%u.
	   Two of these codes toggle underlining and overlining on and off,
	   and therefore take up no visual space , so they must be handled
	   differently.  The other four take up one visual space.
	 */

	if (pnding == -1) {
	    switch(my_char) {
	    case 111:		      /* overline */
		pt_ol = (pt_ol == 0) ? 1 : 0; /* set flag */
		pnding = nil;
		break;
	    case 117:		      /* underline */
		pt_ul = (pt_ul == 0) ? 1 : 0; /* set flag */
		pnding = nil;
		break;
	    default:
		pnding = nil;
		++loc.vsp;
		break;
	    }
	    ++loc.rsp;
	} else if (pnding == 0) {
	    ++loc.rsp;
	    ++loc.vsp;
	    loc.nch = 1;
	} else {
	    ++loc.rsp;
	    loc.nch = 1;
	}
    }
    return loc.vsp;
}

/*  PARSE_STRING_TO_VISUAL_CHARS -- Parse the string "str" such that
    each visual character or control character set is counted as one
    or zero visual characters.	 Place these values into the local
    structure loc, where loc.vsp is the visual character position
    within the string, loc.rsp is the character position in the string
    at the final character of the visual string position, and loc.nch
    is the number of characters comprising the visual string position.

    Return the local structure loc.
 */
struct strloc
/*FCN*/parse_str_to_visual_chars(str, maxv)
  char *str;
  int maxv;			      /* Maximum visual position */
{
    register rsp, k;
    int vsp, nch, sl;
    struct strloc loc;
    char *jstr, j[256], my_chr;

    vsp = rsp = k = 0;
    nch = 1;
    jstr = j;
    strcpy(jstr, str);
    sl = strlen(str);

    /* Now, get the character in the array `j' at `rsp' for all five
       possible lengths of control chacters in an AutoCAD text string.
       Increment `rsp' through as many as required.
      */
    do {			      /* do while (vsp < maxv); */
        if (j[rsp] == '%') {          /* If first character is `%'... */
	    rsp++;		      /* increment rsp and check next char. */
            if (j[rsp] == '%') {      /* If second character is `%'... */
		rsp++;		      /* increment rsp and check next char. */
                if (j[rsp] == ' ') {  /* -- third character is a space. */
		    if (rsp != sl-1) {/* if embedded within a string... */
			rsp++;
			nch = 3;
			vsp++;
		    } else {	      /* NOT embedded within a string... */
			rsp++;
			vsp = maxv;
			nch = 1;
		    }
		    /* -- third character is a */
                } else if (((my_chr = tolower(j[rsp])) == '%') ||
                           (my_chr == /*MSG48*/'d') ||    /* or degrees */
                           (my_chr == /*MSG49*/'p') ||    /* plus/minus */
                           (my_chr == /*MSG50*/'c')) {    /* diameter */
		    rsp++;
		    vsp++;
		    nch = (vsp == maxv) ? 1 : 3;
		    /* -- third character is an... */
                } else if ((my_chr == /*MSG51*/'o') ||    /* overline */
                           (my_chr == /*MSG52*/'u')) {    /* underline */
		    rsp++;
		    /* -- third character is a numeral... */
		} else if (isdigit(my_chr)) {
		    /* check that two more follow... */
		    for (k=0;k<3; ) {
			if (isdigit(j[rsp]) + k) {
			    k++;
			} else {
			    rsp += k;
			    nch = k + 2;
			    k = 3;
			}
		    }
		    vsp++;
                } else if (j[rsp] == '\0') {
		    maxv = vsp;
		} else {	      /* Unknown 3 char. control string. */
		    rsp++;
		    nch = 3;
		}
            } else if (j[rsp] == '\0') {
		maxv = vsp;
	    } else {		      /* Second char is a simple char. */
		rsp++;		      /* probably an error in the string. */
		vsp++;
		nch = 1;
	    }
        } else if (j[rsp] == '\0') {
	    maxv = vsp;
	} else {		      /* First char is a simple char. */
	    rsp++;
	    vsp++;
	    nch = 1;
	}
    } while (vsp < maxv);

    loc.rsp = (int)rsp;
    loc.vsp = vsp;
    loc.nch = nch;
    return loc;
}

/*  PARSE_STRING_TO_REAL_CHARACTERS -- Parse the string "str" such that
    each visual character or control character set is counted as one or
    zero visual characters.   Place these values into the local structure
    loc, where loc.vsp is the visual character position within the string,
    loc.rsp is the character position in the string at the final character
    of the visual string position, and loc.nch is the number of characters
    comprising the visual string position.

    Return the local structure loc.
 */
struct strloc
/*FCN*/parse_str_to_real_chars(str, maxr)
  char *str;
  int maxr;			      /* Maximum real position */
{
    register rsp, k;
    int vsp, sl;
    struct strloc loc;
    char *jstr, j[256], my_chr;

    vsp = rsp = k = 0;

    jstr = j;
    strcpy(jstr, str);
    sl = strlen(str);

    if (maxr == 1) {
	loc.rsp = 1;
	loc.vsp = 1;
	loc.nch = 1;
    } else {

	while (rsp < maxr) {
            if (j[rsp] == '%') {
		rsp++;
                if (j[rsp] == '%') {
		    rsp++;
                    if (j[rsp] == ' ') {    /* space */
			rsp++;
			if (rsp == sl) {
			    vsp += 2;
			} else {
			    vsp++;
			}
                    } else if (((my_chr = tolower(j[rsp])) == '%') ||
                               (my_chr == /*MSG53*/'d') ||    /* or degrees */
                               (my_chr == /*MSG54*/'p') ||    /* plus/minus */
                               (my_chr == /*MSG55*/'c')) {    /* diameter */
			rsp++;
			vsp++;
                    } else if ((my_chr == /*MSG56*/'o') ||    /* overline */
                               (my_chr == /*MSG57*/'u')) {    /* underline */
			rsp++;
			/* -- third character is a numeral... */
		    } else if (isdigit(my_chr)) {

			for (k=0;k<3; ) {
			    if (isdigit(j[rsp]) + k) {
				k++;
			    } else {
				rsp += k;
				k = 3;
			    }
			}
			vsp++;
		    } else {
			rsp++;
		    }
		} else {
		    rsp++;
		    vsp++;
		}
	    } else {
		rsp++;
		vsp++;
	    }
	}
	if (vsp > maxr) {
	    vsp--;
	    rsp--;
	}

	loc.rsp = (int)rsp;
	loc.vsp = vsp;
	loc.nch = 1;		      /* Dummy value */
    }

    return loc;
}

/*  MOVE_CURSOR -- Move the cursor the direction "dir" and if the second
    argument is non-zero, then replace the character under the new cursor
    location with a blank.

    Return the new real string position.
 */
int
/*FCN*/move_cursor(dir, dstrct)
  enum direction dir;
  int dstrct;
{

    struct e_handles *ehtemp, *ehsave;
    struct resbuf rb, *te;
    char *tstr, t[256], *s1;
    int j, n_back = 0;
    struct strloc tloc;

    tstr = t;
    ehtemp = NULL;

    strcpy(tstr, str);

    switch (dir) {
    case LEFT:
	if (dstrct) {		      /* Deleting text -- Backspace key */
	    if (loc.vsp > 1) {	      /* If not at the beginning of a line */
		/* subtract one visual character from the current position. */
		loc = parse_str_to_visual_chars(tstr, --loc.vsp);
		strncpy(tstr, str, loc.rsp-loc.nch);
                tstr[loc.rsp-loc.nch] = '\0';

		s1 = str+loc.rsp;

		/* Add the rest of the old string. */
		strcat(tstr, s1);
		strcpy(str, tstr);
		for (j=0;j<loc.nch;j++) {
                    ads_printf("%c %c", P_BACK, P_BACK);
		}
	    } else {
		/* AT the beginning of the text line... */
		if (pt_cl > 1)	      /* if not at the first line... */
		    /* back up a line, destructive */
		    if (delete_a_line(TRUE) != RTNORM)
			return 0;
	    }
	} else {
	    if (loc.vsp > 1) {	      /* NOT deleting text... ^L */
		tloc = parse_str_to_visual_chars(tstr, --loc.vsp);
	    } else if (pt_cl > 1) {   /* if not at the first line... */
		/* back up a line, non-destructive */
		if (delete_a_line(nil) != RTNORM)   /* back up a line */
		    return 0;
	    }
	}
	break;
    case RIGHT:
	if (dstrct) {		      /* ^A (append) -- overwriting text */
	    if (loc.vsp < pt_msp) {   /* if not at the end of a line */
		tloc = parse_str_to_visual_chars(tstr, loc.vsp++);
		strncpy(tstr, str, tloc.rsp);
                tstr[tloc.rsp] = '\0';

                strcat(tstr, " ");    /* Add a space */

		s1 = str+tloc.rsp+1;

		/* Add the rest of the old string. */
		strcat(tstr, s1);
		strcpy(str, tstr);
	    }
	}
	if (loc.vsp < pt_msp) {       /* NOT deleting text... ^R */
	    /* NOT at the end of a line... */
	    tloc = parse_str_to_visual_chars(tstr, loc.vsp++);
	} else if (pt_cl < pt_mnl) {  /* && AT the end of a line... */
	    /* If not at the last line in the edit list... */
	    /* Modify the current entity to remove the cursor. */
	    rb.restype = 1;
	    rb.resval.rstring = (char *)malloc((unsigned)strlen(str)+1);
	    if (!rb.resval.rstring) {
		ads_fail(sstrs[(int)AERR].sstr);
		return 0;
	    }
	    strcpy(rb.resval.rstring, str);

	    /* Modify the current entity to remove the cursor. */
	    if (pt_mod_ent(pt_te, rb.restype, &rb) != RTNORM)
		return 0;

	    pt_cl++;		      /* add one to current line counter */
	    /* get the ename from PT_LST for the new current line. */
	    eh = (struct e_handles *)qnext(&eh->dllist,&ehhead->dllist);
	    /* Get the entity name */
	    if (ads_handent(eh->hand, pt_te) != RTNORM)
		return 0;

	    /* Retrieve entity data */
	    if ((ce = ads_entget(pt_te)) == NULL)
		return RTERROR;
	    set_text_params(ce);
	    cb = assoc_rb(ce, 1);
	    strcpy(str, cb->resval.rstring);
	    loc.vsp = 1;
	}
	/* Else do nothing. */
	break;
    case HOME:			      /* ^B */
	loc.vsp = 1;
	break;
    case END:			      /* ^E */
	loc.vsp = pt_msp;
	break;
    case TOP:			      /* ^T */
	rb.restype = 1;
	rb.resval.rstring = (char *)malloc((unsigned)strlen(str)+1);
	if (!rb.resval.rstring) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}
	strcpy(rb.resval.rstring, str);
	n_back = strlen(str);

	/* Modify the current entity to remove the cursor. */
	if (pt_mod_ent(pt_te, rb.restype, &rb) != RTNORM)
	    return 0;
	pt_cl = 1;
	eh = ehhead;
	/* Get the entity name */
	if (ads_handent(eh->hand, pt_te) != RTNORM)
	    return 0;

	/* Retrieve entity data */
	if ((ce = ads_entget(pt_te)) == NULL)
	    return RTERROR;
	set_text_params(ce);
	cb = assoc_rb(ce, 1);	      /* Retrieve entity string */
	strcpy(str, cb->resval.rstring);
	n_back -= strlen(str);

	loc.vsp = 1;
	break;
    case BOTTOM:		      /* ^N */
	rb.restype = 1;
	rb.resval.rstring = (char *)malloc((unsigned)strlen(str)+1);
	if (!rb.resval.rstring) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}
	strcpy(rb.resval.rstring, str);
	n_back = strlen(str);

	/* Modify the current entity to remove the cursor. */
	if (pt_mod_ent(pt_te, rb.restype, &rb) != RTNORM)
	    return 0;

	pt_cl = pt_mnl;
	ehtemp = eh;
	/* Get the last entity handle in the linked list. */
	eh = (struct e_handles *)qlast(&ehhead->dllist);
	if (eh == NULL) {
	    eh = ehtemp;
	} else {
	    /* Get the entity name */
	    if (ads_handent(eh->hand, pt_te) != RTNORM)
		return 0;

	    /* Retrieve entity data */
	    if ((ce = ads_entget(pt_te)) == NULL)
		return RTERROR;
	    set_text_params(ce);
	    cb = assoc_rb(ce, 1);
	    strcpy(str, cb->resval.rstring);
	    n_back -= strlen(str);

	    tloc = parse_str_to_real_chars(str, strlen(str));
	    loc.vsp = tloc.vsp;
	}
	break;
    case DELETE:		      /* ^Delete key or ^H */
	if (loc.vsp < pt_msp) {       /* if not at the end of the line... */
	    strncpy(tstr, str, loc.rsp-loc.nch);
            tstr[loc.rsp-loc.nch] = '\0';
	    s1 = str+loc.rsp;

	    /* Add the rest of the old string. */
	    strcat(tstr, s1);

            /* Truncate "tstr" to 255 chars + trailing NULL */
            if (strlen(tstr) > 255) tstr[255] = '\0';

            /* Copy it back into "str" */
            str[0] = '\0';
	    strcpy(str, tstr);

	    for (j=0;j<loc.nch;j++) {
                ads_printf("%c %c", P_BACK, P_BACK);
	    }
	} else {
	    /* at the last character in the line... */
	    /* if it is a blank...		    */
            if (str[loc.rsp-1] == ' ') {
		if (qnext(&eh->dllist,&ehhead->dllist) != NULL) {
		    ehtemp = (struct e_handles *)qnext(&eh->dllist,
						       &ehhead->dllist);
		    /* Get the entity name */
		    if (ads_handent(ehtemp->hand, pt_temp) != RTNORM)
			return 0;
		    /* Retrieve entity data */
		    if ((te = ads_entget(pt_temp)) == NULL)
			return RTERROR;
		    cb = assoc_rb(te, 1);
                    str[loc.rsp-loc.nch] = '\0';

		    strcat(str, cb->resval.rstring);
		    ehtemp = find_entity_handle(ehtemp->hand);

		    if (ehtemp == ehhead) {
			ehhead = (struct e_handles *)qnext(&ehtemp->dllist,
							   &ehhead->dllist);
		    }

		    ehsave = eh;
		    eh = ehtemp;

		    /* Bring up the lines following this line. */
		    if (pt_move_lines(UP, TRUE) != RTNORM)
			return 0;

		    /* eh = ehsave;
		    ehtemp = ehsave; */

		    ads_entdel(pt_temp);

		    qremove(&ehtemp->dllist);
		    pt_mnl--;

		    /* if (qprev(&ehtemp->dllist,&ehhead->dllist) != NULL)
			eh = (struct e_handles *)qprev(&ehtemp->dllist,
						       &ehhead->dllist); */
		    eh = ehsave;
		}
	    } else {
		/* subtract one visual character from the current position. */
		strncpy(tstr, str, loc.rsp-loc.nch);
                tstr[loc.rsp-loc.nch] = '\0';

		/* replace the current character with a space. */
                strcat(tstr, " ");

                /* Copy it back into "str" */
                str[0] = '\0';
		strcpy(str, tstr);
	    }
	}
	break;
    case UP:			      /* ^U */
    case DOWN:			      /* ^D */
	/* Never let pt_cl below 1 or above length of list PT_LST. */
	if ((pt_cl > 0) && (pt_cl <= pt_mnl)) {
	    rb.restype = 1;
	    rb.resval.rstring = (char *)malloc((unsigned)strlen(str)+1);
	    if (!rb.resval.rstring) {
		ads_fail(sstrs[(int)AERR].sstr);
		return 0;
	    }
	    strcpy(rb.resval.rstring, str);
	    n_back = strlen(str);

	    /* Modify the current entity to remove the cursor. */
	    if (pt_mod_ent(pt_te, rb.restype, &rb) != RTNORM)
		return 0;

	    ehtemp = eh;
	    if ((dir == UP) && (pt_cl > 1)) {
		eh = (struct e_handles *)qprev(&eh->dllist,&ehhead->dllist);
		pt_cl--;
	    } else if ((dir == DOWN) && (pt_cl < pt_mnl)) {
		eh = (struct e_handles *)qnext(&eh->dllist,&ehhead->dllist);
		pt_cl++;
	    }
	    if (eh == NULL) {
		eh = ehtemp;
	    } else {
		/* get the ename from eh->hand for the new current line. */
		/* Get the entity name */
		if (ads_handent(eh->hand, pt_te) != RTNORM)
		    return 0;

		/* Retrieve entity data */
		if ((ce = ads_entget(pt_te)) == NULL)
		    return RTERROR;
		set_text_params(ce);
		cb = assoc_rb(ce, 1);
		strcpy(str, cb->resval.rstring);
		n_back -= strlen(str);
	    }

	}
	break;
    default:
        ads_fail(/*MSG58*/"Unknown code");
	return 0;
    }
    for (j=0;j<n_back;j++) {
        ads_printf("%c %c", P_BACK, P_BACK);
    }
    ads_printf("\r%s%s", sstrs[(int)TEXT].sstr, str);
    return loc.rsp;
}

/*  DELETE_A_LINE -- Delete a line and/or back up to the previous one. */
int
/*FCN*/delete_a_line(temp)
  int temp;			      /* If temp is non-zero, then go ahead */
{				      /* and delete, else, just back up...  */

    struct resbuf *rb;
    struct e_handles *ehtemp;
    ads_real angle;

    int j, sl, ent, grp_72;

    sl = strlen(str);
    strcpy(tstr, str);

    if (temp) {
	if (loc.vsp != pt_msp) {
	    strcpy(tstr, str);
	}
	rb = assoc_rb(ce, -1);
	if (rb)
	    if (ads_entdel(rb->resval.rlname) != RTNORM)
		return 0;

	pt_mnl--;

	for (j=0;j<sl;j++) {
            ads_printf("%c %c", P_BACK, P_BACK);
	}
        ads_printf(/*MSG59*/"*Deleted*");
        ads_printf("\n%s", sstrs[(int)TEXT].sstr);
    } else {

	cb = assoc_rb(ce, 1);
	if (strlen(cb->resval.rstring) < sl) {
	    cb->resval.rstring = (char *) realloc(cb->resval.rstring,
						  (unsigned) sl+1);
	}
	if (!cb->resval.rstring) {
	    ads_fail(sstrs[(int)AERR].sstr);
	    return 0;
	}
	cb->resval.rstring = str;
	if (ads_entmod(ce) != RTNORM) {
	    ads_relrb(ce);
	    return 0;
	}
    }
    str[0] = '\0';

    /* Remove last entity from linked list if at the maximum string  */
    /* position discounting control characters and both are 1. */

    ehtemp = eh;
    if (ehtemp == ehhead) {
	ehhead = (struct e_handles *)qnext(&ehtemp->dllist, &ehhead->dllist);
    }

    /* Get the entity name */
    if (pt_cl > 0) {
	if (qprev(&eh->dllist,&ehhead->dllist) != NULL) {
	    eh = (struct e_handles *)qprev(&eh->dllist,&ehhead->dllist);
	} else if (pt_cl == 2) {
	    eh = ehhead;
	}
	if ((ent = ads_handent(eh->hand, pt_te)) != RTNORM) {
            ads_fail(/*MSG60*/"Invalid entity handle");
	}
    } else {
	ent = 0;
    }

    if (temp)
	qremove(&ehtemp->dllist);

    if (ent > 0) {

	/* Retrieve entity data */
	if ((ce = ads_entget(pt_te)) == NULL)
	    return RTERROR;
	rb = assoc_rb(ce, 1);
	str = s;
	strcpy(str,rb->resval.rstring);

	loc = parse_str_to_real_chars(str, strlen(str));
	if ((tstr) && (strlen(tstr) > 0)) {
	    loc.vsp++;		      /* Set cursor to position  */
	    /* following end of string */
	}

	if (temp) {
	    /* Bring up the lines following this line. */
	    if (pt_move_lines(UP, TRUE) != RTNORM)
		return 0;
	    strcat(str, tstr);
            /* Truncate "str" to 255 chars + trailing NULL */
	    if ((sl + strlen(tstr)) > 255)
                str[255] = '\0';
	}

	/* set maximum string postion (pt_msp) after allowing for */
	/* various control character codes such as %%d or %%p.	  */
	tloc = parse_str_to_real_chars(str, strlen(str));
	pt_msp = tloc.vsp;

	angle = pt_ra + (M_PI / 2);
	ads_polar(pt_spt, angle, pt_ils, spm->pl);
	copy_pt(pt_spt, spm->pl);

	set_text_params(ce);

        ads_printf("%s", str);

	rb = assoc_rb(ce, 72);
	grp_72 = rb->resval.rint;

	if (grp_72 == 5) {
	    ads_polar(pt_rpt, angle, pt_ils, spm->pr);
	    copy_pt(pt_rpt, spm->pr);
	}
    }
    pt_cl--;

    return RTNORM;
}

/*  PT_MOVE_LINES -- Pick up all of the lines of text following
    the current line and move them either UP or DOWN, based on
    the passed in value of `dir'.
*/

int
/*FCN*/pt_move_lines(dir, flag)
  enum direction dir;
  int flag;
{

    struct e_handles *ehtemp;
    ads_real angle = 0;
    ads_name sset, ename;
    long len;

    if (dir == UP) {
	angle = pt_ra + (M_PI / 2);
    } else if (dir == DOWN) {
	angle = pt_ra - (M_PI / 2);
    }

    ads_ssadd(NULL, NULL, sset);

    ehtemp = eh;

    while (qnext(&eh->dllist,&ehhead->dllist) != NULL) {
	eh = (struct e_handles *)qnext(&eh->dllist,&ehhead->dllist);
	if (ads_handent(eh->hand, ename) != RTNORM)  /* Get the entity name */
	    return 0;
	if (ads_ssadd(ename, sset, sset) != RTNORM) {
            ads_fail(/*MSG61*/"\nUnable to add to selection set. ");
	    return 0;
	}
    }
    eh = ehtemp;

    if (sset) {
	ads_sslength(sset, &len);
	ads_polar(pt_spt, angle, pt_ils, pt_spt2);
	if (len > 0) {

            ads_command(RTSTR, /*MSG62*/"_.move",
			RTPICKS, sset,
                        RTSTR, "",
			RT3DPOINT, pt_spt,
			RT3DPOINT, pt_spt2,
			0);
	    /* copy_pt(pt_spt, pt_spt2); */
	}
	if (flag) {
	    copy_pt(spm->pl, pt_spt2);
	    copy_pt(pt_spt, pt_spt2);
	}

	if (sset) ads_ssfree(sset);
    }
    return RTNORM;
}

int temp_vsp, save_vsp, ext_flg;

/*  WRAP_AT_WORD -- Wrap the line at the end of the previous word,
    if there is one.  Otherwise, if the line is short and/or the word
    is long enough to occupy the entire line length, then extend the word.
 */
int
/*FCN*/wrap_at_word()
{

    struct resbuf *cb;
    ads_real line_l2;
    char t2[256], *str2;

    str2 = t2;

    /* Retrieve entity data */
    if ((ce = ads_entget(pt_te)) == NULL)
	return RTERROR;

    set_text_params(ce);

    /* Get the justification type of the entity */
    cb = assoc_rb(ce, 72);
    grp_72 = cb->resval.rint;

    if (grp_72 != 5) {
	/* Get the new string position. */
	temp_vsp = get_maximum_string_position();
	tloc = parse_str_to_real_chars(str, strlen(str));
	save_vsp = tloc.vsp - loc.vsp;
	if (save_vsp < 0) save_vsp = 0;
	loc = parse_str_to_visual_chars(str, temp_vsp);
    } else {
	temp_vsp = 0;
    }

    if (grp_72 == 5) {		      /* Fit text... */
	/* Set up to test if we are within the last 1/4 of the word, */
	/* and if so, we will cram the whole word on the line. */
	strcpy(str2, str);	      /* Save the value of str */

	pt_strrev(tstr, str);	      /* Reverse the string. */
        tstr = strtok(tstr, " ");     /* Get first word (actually last). */
	pt_strrev(str, tstr);	      /* Reverse it back. */
        s[strlen(tstr)] = '\0';       /* Terminate the substring. */

	line_l2 = get_line_length(loc.rsp);

	strcpy(str, str2);	      /* Restore the value of str */
	if ((line_l - (line_l2 / 4)) < pt_mll) {
	    line_l = 0.0;
	}

    } else {
	/* Else, set a dummy value large enough to trip the next test. */
	line_l *= 2.0;
    }
    if (ext_flg) {
	return loc.vsp;
    } else {
	return loc.vsp++;	      /* Return the visual string position. */
    }
}

/*  STRING_REVERSE -- Reverse a string excepting the trailing null. */
void
/*FCN*/pt_strrev(t, s)
  char *t;
  char *s;
{
    int sl, i;

    sl = strlen(s);

    for(i=0;i<=sl;(t[i] = s[sl-i])) i++;
    return;
}

/*  GET_MAXIMUM_STRING_POSITION -- Get the maximum string position
    allowed on a line given the current contents of the variable "str".
    Uses get_line_length().

    Get a temporary projected number of characters allowed on a line.
    Check this against the actual line length of the real characters,
    adding one character until either the end of the string is reached
    or the maximum line length is reached.  Then start subtracting
    characters until either a space or hypen is found, or the start of
    the string is reached.  If the start of the string is reached, then
    search forward on the string looking for the end of the word.  If
    the end of the word is not found then return the string length, else
    return the wrap position on the line.
 */
int
/*FCN*/get_maximum_string_position()
{

    struct strloc tloc;

    int temp, stemp, sl;

    tstr = t;
    strcpy(tstr, str);
    sl = strlen(str);

    /* Make a guess at the correct number of characters... */
    stemp = temp = round(pt_mll / (0.8 * pt_twf * pt_th));

    tloc = parse_str_to_visual_chars(str, temp);

    if (tloc.vsp > loc.vsp) {
	stemp = temp = loc.vsp;
    }

    /* Add characters , then check the line length, until... */
    while ((tloc.vsp < loc.vsp) && (get_line_length(tloc.rsp) < pt_mll)) {
	temp++;
	tloc = parse_str_to_visual_chars(str, temp);
    }
    /* Check the line length, , then subtract characters until... */
    while (get_line_length(tloc.rsp) > pt_mll) {
	temp--;
	tloc = parse_str_to_visual_chars(str, temp);
    }
    if (tloc.rsp >= sl) {
	tloc.rsp = sl-1;
    }
    while ((tloc.vsp > 1) &&
           (t[tloc.rsp-tloc.nch] != ' ') &&   /* Back up until a space */
           (t[tloc.rsp-tloc.nch] != '-')) {   /* or hyphen is found... */
	temp--;
	tloc = parse_str_to_visual_chars(str, temp);
    }
    if (temp == 1) {
	temp = stemp-1;
	tloc = parse_str_to_visual_chars(str, temp);
	while ((temp <= loc.vsp) &&
               (t[tloc.rsp-tloc.nch] != ' ') && /* Go forward until a space */
               (t[tloc.rsp-tloc.nch] != '-')) { /* or hyphen is found... */
	    temp++;
	    tloc = parse_str_to_visual_chars(str, temp);
	}
    }
    if (temp > loc.vsp) {
	ext_flg = FALSE;
	temp = loc.vsp;
	return temp;
    } else if (temp == loc.vsp) {
	ext_flg = TRUE;
	return temp;
    } else {
	ext_flg = FALSE;
	return temp;
    }
}

/*  PROCESS_WORD_WRAP -- Process word wrap. */
int
/*FCN*/process_word_wrap()
{

    struct resbuf *rb;

    /* Not at start of a line and line is longer than maximum specified.*/
    if ((loc.rsp > 1) && (line_l > pt_mll) && (ext_flg != TRUE)) {
	/* Wrapping a text line... */
	/* Process existing entity... */
	if (process_existing_entity() != RTNORM)
	    return 0;

	str = s;
        /* s[0] = '\0'; */

	if (pt_ol) {
            strcpy(str, "%%o");
	    if (pt_ul) {
                strcat(str, "%%u");
	    }
	    strcat(str, tstr+loc.rsp);
	} else if (pt_ul) {
            strcpy(str, "%%u");
	    strcat(str, tstr+loc.rsp);
	} else {
	    strcpy(str, tstr+loc.rsp);
	}

	if (strlen(str) == 0) {
            strcpy(str, " ");
	    loc.rsp = 1;
	    loc.vsp = 1;
	    loc.nch = 1;
	    pt_msp  = 1;
	}


	tloc = parse_str_to_real_chars(str, strlen(str));
	loc = parse_str_to_visual_chars(str, (tloc.vsp - save_vsp));

	/* set maximum string postion (pt_msp) after allowing for */
	/* various control character codes such as %%d or %%p.	  */
	pt_msp = tloc.vsp;
	if (pt_move_lines(DOWN, TRUE) != RTNORM)
	    return 0;

	if (grp_72 == 5) {	      /* if Fit text */

	    rb = assoc_rb(ce, 11);
	    copy_pt(pt_rpt, rb->resval.rpoint);

	    ads_polar(pt_rpt, pt_ra - (M_PI / 2), pt_ils, spm->pr);

	}

	set_text_params(ce);

	ce = NULL;

	/* Make the new text line here... */
	/* set string that gets printed on-screen via (ads_entmod).    */
	tstr = underline_string(str, loc.rsp, insert);



	if (process_line(tstr) != RTNORM)    /* Process the line */
	    return 0;

    } else {			      /* Extending a text line... */

	/* set the string that gets printed on-screen via (entmod). */
	tstr = underline_string(str, loc.rsp, insert);

	if (process_line(tstr) != RTNORM)    /* Process the line */
	    return 0;

	ext_flg = FALSE;

    }
    ads_printf("\n%s%s", sstrs[(int)TEXT].sstr, str);
    return RTNORM;
}

/*  PROCESS_EXISTING_ENTITY -- Process the existing string,
    entmoding it to its final form.
 */
int
/*FCN*/process_existing_entity()
{

    struct resbuf *rb, rb1;
    register j;
    int grp_72;

    rb = &rb1;

    for (j=0;j<(pt_msp-loc.rsp);j++) {
        ads_printf("%c %c", P_BACK, P_BACK);
    }

    strcpy(tstr, str);
    strncpy(str, tstr, loc.rsp);
    str[loc.rsp] = '\0';

    /* Strip trailing space. */
    if (str[loc.rsp-1] == ' ') {
        str[loc.rsp-1] = '\0';
    }

    ads_printf("\r%s%s", sstrs[(int)TEXT].sstr, str);

    if (ce) {
	rb->restype = 1;
	rb->resval.rstring = str;
	/* Modify the current entity to remove the cursor. */
	if (pt_mod_ent(pt_te, rb->restype, rb) != RTNORM)
	    return 0;

	pt_cl++;

        /* Get correct "start point" for current type of text entity...  */
	/* This should correctly handle mixed text justification types.  */

	rb = assoc_rb(ce, 72);
	grp_72 = rb->resval.rint;

	if ((grp_72 == 2) || (grp_72 == 1)) {

	    rb = assoc_rb(ce, 11);
	    copy_pt(pt_spt, rb->resval.rpoint);

	} else {

	    rb = assoc_rb(ce, 10);
	    copy_pt(pt_spt, rb->resval.rpoint);
	}


    }
    return RTNORM;
}
