/*

    AToolbox

    A Windows/ADS Application: Modeless Icon Tool Palette
      by Phil Ford, Apr 21, 1992

    (xload "atoolbox")
    (tboxdlg) Run Toolbox Dialog
    (tbox 0)  Drawing Toolbox
    (tbox 1)  Icon Screen Menu
    (tbox 2)  Text Button Screen Menu
    (tbox_quit 0)  Remove Drawing Toolbox
    (tbox_quit 1)  Remove Icon or Text Screen Menu
    (tbox_quit)    Remove all tool boxes

    Funtions:

	tbox		Activate one of array of modeless
			tool palettes.  Arg: 1 int, the
			toolbox number.
	tbox_quit	Remove a certain toolbox or all toolboxes


     ________________________________________________________________________

      (C) Copyright 1992-93 by Autodesk, Inc.

      Permission to use, copy, modify, and distribute this software and its
      documentation for any purpose and without fee is hereby granted.

      THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
      ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
      MERCHANTABILITY ARE HEREBY DISCLAIMED.
     ________________________________________________________________________


*/

#include "windows.h"
#include "adslib.h"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>
#include "windde.h"
#include "tbox.h"
#include "atboxdlg.h"



typedef struct {
    char *cmdname;
    void (*cmdfunc)();
} CMDTAB;
#define ELEMENTS(array) (sizeof(array)/sizeof((array)[0]))

WORD wm_acad;


/* High Level Function Prototypes for atoolbox.c */

void main(int argc, char **argv);
static short funcload(void);
static void tboxdlg(void);
static void tbox(void);
static void tbox_quit(void);

/* Table of AutoCAD function (or command) names
   and corresponding C function name
*/
CMDTAB cmdtab[] = {
    /*   Name    Function  */
    {"tboxdlg", tboxdlg},
    {"tbox", tbox},
    {"tbox_quit", tbox_quit},
};




/*

		  AToolbox Main

*/


static int sendAcad(char *func);
static FARPROC lpfnParentSubProc;
static FARPROC lpfnParentProc = NULL;

long FAR PASCAL ParentSubProc(HWND hwnd, unsigned msg, WORD wParam,
                                LONG lParam);
BOOL FAR PASCAL ATBOXDLGMsgProc(HWND hWndDlg, WORD Message,
                                WORD wParam, LONG lParam);

void main(int argc, char **argv)
{
    short scode = RSRSLT;
    short cindex;
    short ads_req;


    ads_init(argc, argv);

    /* Get special message with which to send AutoCAD commands */
    wm_acad = RegisterWindowMessage("WM_ACAD");

#ifdef SHOW_AS_ICON
    ShowWindow(adsw_hWnd, SW_SHOWMINNOACTIVE);
    UpdateWindow(adsw_hWnd);
#endif

    tboxInit(adsw_hInstance, adsw_hwndAcad);

    for ( ;; ) {

	/* This process will block (stop) here until AutoCAD has a
	   request for us */

	ads_req = ads_link(scode);
	if (ads_req < 0)
	    break;

	scode = RSRSLT; 	      /* default return code */

	/* Check for AT LEAST the following cases here */
 	switch (ads_req) {

	case RQXLOAD:          /* Load request. Send function names. */
	    scode = -(funcload() ? RSRSLT : RSERR);
	    break;

	/* Execute a "loaded" function that was defined via RQXLOAD */
	case RQSUBR:
	    cindex = ads_getfuncode();

	    if (cindex >= 0 && cindex < ELEMENTS(cmdtab))
		(*cmdtab[cindex].cmdfunc)();

	    break;

	/* (xunload "adsw_AppName") entered or called in AutoCAD */
	case RQXUNLD:
	    tboxQuitAll();
	    break;

	default:
	    break;
	}
    }		/* end of ADS loop */

}





/* Register our functions with the ADS system.  Also,
   any drawing initialization can go here, since this is
   called with each new drawing.
*/
static short funcload(void)
{
    int idx;
    char *cname;

    for (idx = 0; idx < ELEMENTS(cmdtab); ++idx) {
	cname = cmdtab[idx].cmdname;
	ads_defun(cname, idx);
    }
    return TRUE;
}



/* AutoCAD function to display the toolbox dialog. */

static void tboxdlg(void)
{
    FARPROC lpfnATBOXDLGMsgProc;
    int nRc;

    lpfnATBOXDLGMsgProc = MakeProcInstance((FARPROC)ATBOXDLGMsgProc,
                                adsw_hInstance);
    nRc = DialogBox(adsw_hInstance, (LPSTR)"ATBOXDLG", adsw_hWnd,
                                        lpfnATBOXDLGMsgProc);
    FreeProcInstance(lpfnATBOXDLGMsgProc);
}



/* AutoCAD function to display a certain toolbox.  One parameter:
   integer, index of toolbox. */

static void tbox(void)
{
    int index = 0;
    struct resbuf *rb;

    rb = ads_getargs();
    if (rb != NULL) {
        if (rb->restype != RTSHORT) {
	    ads_printf("Argument must be an integer\n");
	    return;
	} else
	    index = rb->resval.rint;
    }
    if (adsw_hwndAcad != (HWND)0) {
        switch (index) {
        case 0:
	    tboxFloat(adsw_hwndAcad, 0);
	    break;
        case 1:
    	    /* tboxLocked(adsw_hwndAcad, 0); */
	    /* Box 0 locked to right, 2 icons wide */	
    	    tboxDraw(adsw_hwndAcad, 0, 0, 0, 2, 25, 0, FALSE);
	    break;
	/*
	case 2:
    	    tboxText(adsw_hwndAcad, 1);
	    break;
	*/
	default:
	    break;
	}
    } else
        adsw_msg_box(0, "AutoCAD Window handle is NULL");
}


/* AutoCAD function to remove a certain toolbox.  One parameter:
   integer, index of toolbox. */

static void tbox_quit(void)
{
    int index = 0;
    struct resbuf *rb;

    rb = ads_getargs();
    if (rb != NULL) {
        if (rb->restype != RTSHORT) {
	    ads_printf("Argument must be an integer\n");
	    return;
	} else {
	    index = rb->resval.rint;
            tboxQuit(index);
	}
    } else
        tboxQuitAll();
}

/* Send command such as "erase " to "interrupt" AutoCAD
   to perform command. */
static int sendAcad(char *func)
{
    SendMessage(adsw_hwndAcad, wm_acad, 0, (LONG)(LPBYTE)func);
}


/*

    Toolbox

    Modeless Icon Tool Palettes

*/



/* Definitions */
#define wBMAP 16                      /* Based on 16x16 bitmaps created with
					 an image editor */
#define nxTOOLS 4                     /* Set bitmap images x per row */
#define wEDGEDEF 2                    /* Default edge width of button */
#define wEDGE 2 		      /* Edge width of button */
#define yTOOL (wBMAP+wEDGE+wEDGE)     /* Leave room for 3-d shading */
#define wFRAME 4		      /* width of border between window frame and
 				         buttons */
#define wEXTRA 8		      /* make width of each button this much more
				         than the height */
#define xTOOL (yTOOL+wEXTRA)	      /* buttons look nice a little wider
					 than tall */

/* Button border.  Increase to move button in around edges, leaving a light gray
   border around button. */
#define wBORDER 0
#define XDEFPOS 10
#define YDEFPOS 30


#define Width(r)  (r.right - r.left)
#define Height(r) (r.bottom - r.top)
#define PWidth(r)  (r->right - r->left)
#define PHeight(r) (r->bottom - r->top)

#define RGB_LTGRAY (RGB(192,192,192))
#define RGB_DKGRAY (RGB(130,130,130))
#define RGB_WHITE  (RGB(255,255,255))
#define RGB_BLACK  (RGB(  0,  0,  0))

/* tool icons */

typedef struct {
    char *bmap_name;
    char *command;
    char *text;
    RECT rect;
} TOOL;



/* toolbox */

typedef struct {
    char *title;		      /* Toolbox title bar text */	
    short nTool;		      /* Total number of tools in box */	
    TOOL *tool;			      /* ptr to array of tools */
    short nToolX;		      /* number of tools in row */	
    short nToolY;		      /*	            col */
    RECT rect;			      /* toolbox rect relative to owner */	
    short top;			      /* Offset from top of client area
     					 of parent window */	
    short right;		      /* Offset in from right */		
    HWND hwnd;			      /* toolbox window handle */	
    HWND hwndOwner;		      /* owner--main window handle */	
    HANDLE hInstance;		      /* instance handle */	
    DWORD dwStyle;
    struct {
        unsigned text : 1;	      /* Text buttons */	
    } flags;
    short tx;		      	      /* width of each tool in box */	
    short ty;		      	      /* height of each tool in box */	
} TBOX;


/* Array of tools: icons and functions */
static TOOL tDraw[] =  {
    {"Osnap" , "\3osnap ", "Object snap" 	},
    {"None"  , "none ",    "No snap"     	},
    {"Cancel", "\3\3"	   		 	},
    {"Cen"   , "cen ",     "Snap center"        },
    {"End"   , "endp ",    "Snap end"   },
    {"Ins"   , "ins ",     "Snap text insert" },
    {"Int"   , "int ",     "Snap intersection"},
    {"Mid"   , "mid ",     "Snap midpoint"      },
    {"Nea"   , "near ",    "Snap nearest point" },
    {"Per"   , "per ",     "Snap perpendicular" },
    {"Qua"   , "quad ",    "Snap quad"  },
    {"Tan"   , "tan ",     "Snap tangent"      },
    {"Line"  , "\3line ",    "Draw line"        },
    {"Polyl" , "\3pline ",   "Polyline"         },
    {"Polyg" , "\3polygon ", "Polygon"          },
    {"Arc"   , "\3arc ",     "Draw arc"         },
    {"Ellip" , "\3ellipse ", "Ellipse"          },
    {"Circl" , "\3circle ",  "Circle"           },
    {"Dtext" , "\3dtext "                       },
    {"Pedit" , "\3pedit ",   "Pline edit"       },
    {"Move"  , "move "				},
    {"Copy"  , "copy "				},
    {"Scale" , "scale "				},
    {"Rotat" , "rotate ",  "Rotate"   		},
    {"Stret" , "stretch ", "Stretch"  		},
    {"Array" , "array "				},
    {"Erase" , "erase "				},
    {"Divid" , "\3divide ",  "Divide"           },
    {"Mirro" , "mirror ",  "Mirror"   		},
    {"Trim"  , "\3trim "                        },
    {"Extend", "\3extend "                      },
    {"Chamf" , "\3chamfer ", "Chamfer"          },
};

#if NEEDED
    {"Hatch" , "hatch "},
    {"Inser" , "insert "},
    {"Offse" , "offset "},
#endif

#ifdef USE_ACAD_SMENU
/* Array of tools: icons and functions */
static TOOL tAcad[] =  { 
    {"Acad", "\3", "AutoCAD" 			},
    {"Stars", "\3", "****" 			},
    {"ASE", "ase " 				},
    {"BLOCK", "block ", "BLOCKS" 		},
    {"DIM", "dim " 				},
    {"DISP", "display ", "DISPLAY" 		},
    {"DRAW", "draw " 				},
    {"EDIT", "edit " 				},
    {"INQ", "inquiry ", "INQUIRY" 		},
    {"LAYER", "layer " 				},
    {"MODEL", "ame ", "MODELER" 		},
    {"MVIEW", "mview " 				},
    {"PLOT", "plot " 				},
    {"RENDER", "render " 			},
    {"SETTINGS", "settings " 			},
    {"SURFACES", "surfaces " 			},
    {"UCS", "ucs " 				},
    {"UTILITY", "utility " 			},
    {"SAVE", "save " 				},
};
#endif  /* USE_ACAD_SMENU */

/* Toolboxes */
static TBOX tboxAry[] = {
    { "Draw",       ELEMENTS(tDraw), tDraw },
#ifdef USE_ACAD_SMENU
    { "Acad",       ELEMENTS(tAcad), tAcad },
#endif
};

/* Window style names */
enum {
    TB_FIXED = 0,
    TB_FLOAT
};

/* Window style bits */
DWORD styleAry[] = {
    WS_CHILD | WS_VISIBLE, 
    WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_SYSMENU,
};


tboxCreate(TBOX *tbox);
tboxCreateWindow(TBOX *tbox);
toolsCreate(TBOX *tbox);
void tboxShow(TBOX *tbox);
void tboxShowAll(void);
int imageToWin(HWND hwnd, HDC hdc, PAINTSTRUCT *paintst, 
				HANDLE hInstance, char *bmap_name);
int toolsDraw(HDC hdc, TBOX *tbox);
int toolBlit(HDC hDC, HDC hMemDC, TBOX *tbox, TOOL *tool);
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow);
int toolHit(TBOX *tbox, POINT *pt);
int rectHit(RECT *rect, POINT *pt);
void shadeRect(HDC hdc, RECT *rect, int eEdge, int reverse);
long FAR PASCAL tboxProc(HWND hwnd, WORD msg, WORD wParam, LONG lParam);
int toolHitMsg(TBOX *tbox, LONG lParam);
void toolText(TBOX *tbox, int index, int select);
int tboxWidth(TBOX *tbox);
tboxAdjustWin(DWORD dwStyle, int *cx, int *cy);
tboxUpdatePos(TBOX *tbox);
int toolWidth(TBOX *tbox);
int toolDrawText(HDC hdc, TBOX *tbox, TOOL *tool);



/*


	   	T o o l b o x



*/

int CurrentTool;



/* Static Data */
static HANDLE tbInst;
static char ToolboxName[] = "Toolbox";
static int font_ave_width;
static int font_ave_height;

/* Register toolbox class */

int tboxInit(HANDLE Inst, HWND hwnd)
{
    WNDCLASS wndclass;
    TEXTMETRIC fm;
    HDC hdc;

    memset(&wndclass, 0x00, sizeof(WNDCLASS));
    wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_CLASSDC;
    wndclass.lpfnWndProc   = (WNDPROC)tboxProc;
    wndclass.cbClsExtra    = 0;
    wndclass.cbWndExtra    = sizeof(void *);   /* per-window extra data. */
    wndclass.hInstance     = tbInst = Inst;
    wndclass.hIcon         = NULL;
    wndclass.hCursor       = LoadCursor((HANDLE)0, IDC_ARROW);
    wndclass.hbrBackground = GetStockObject(LTGRAY_BRUSH);
    wndclass.lpszMenuName  = NULL;
    wndclass.lpszClassName = ToolboxName;
    if (!RegisterClass (&wndclass)) {
        adsw_msg_box(0, "Error registering toolbox class");
	return FALSE;
    }
    hdc = GetDC(hwnd);
    GetTextMetrics(hdc, &fm);
    font_ave_width = fm.tmAveCharWidth;
    font_ave_height = fm.tmHeight;
    ReleaseDC(hwnd, hdc);
    lpfnParentSubProc = MakeProcInstance((FARPROC)ParentSubProc, 
	    				Inst);
    lpfnParentProc = (FARPROC)SetWindowLong(hwnd, GWL_WNDPROC,
                                        (LONG)lpfnParentSubProc);
    return TRUE;
}




/* 


		A Few Standard Tool Boxes


*/



/* Create an icon toolbox that is locked to the right side of the window */
int tboxLocked(HWND hwnd, int index)
{
    return tboxDraw(hwnd, index, 0, 0, 1, 0, 0, FALSE);
}

/* Create a floating icon toolbox */
int tboxFloat(HWND hwnd, int index)
{
    return tboxDraw(hwnd, index, 1, 0, 4, 100, 300, FALSE);
}


/* Create an text toolbox that is locked to the right side of the window */
int tboxText(HWND hwnd, int index)
{
    return tboxDraw(hwnd, index, 0, 0, 1, 0, 0, TRUE);
}

/* Activate the Draw toolbox.  "style" is 0 for lock to right 
   edge, 1 for floating.  To fit to a certain window width, set
   width.  To set width based on number of tools in a row, set
   nxtools (and set width = 0).  Set both to 0 for default. 
   "top" is offset from top of client area.  "right" is offset
   from right of client area.  Style can be TB_FLOAT or TB_FIXED. */

int tboxDraw(HWND hwndOwner, int index, int style, int width, 
	int nxtools, int top, int right, int textmode)
{
    TBOX *tbox;
    DWORD dwStyle;
    int tx;

    if (index < 0 || index >= ELEMENTS(tboxAry))
        return FALSE;

    if (style >= ELEMENTS(styleAry))
        style = ELEMENTS(styleAry) - 1;

    dwStyle = styleAry[style];

    tbox = &tboxAry[index];
    if (tbox->hwnd != (HWND)0) {
	/* Already up and running.  Caller might want to change
	   shape. */
        DestroyWindow(tbox->hwnd);
	tbox->hwnd = (HWND)0;
    }
 
    tbox->hwndOwner = hwndOwner;
    tbox->hInstance = tbInst;
    tbox->dwStyle = dwStyle;
    tbox->top = top;
    tbox->right = right;
    tbox->flags.text = textmode;
    tbox->tx = tx = toolWidth(tbox);
    tbox->ty = toolHeight(tbox);
    if (width != 0)
	/* Calculate number of icons in x & then y dimension */
        tbox->nToolX = (width - (2 * wFRAME)) / tx;
    else if (nxtools != 0)
        tbox->nToolX = nxtools;
    else   
	tbox->nToolX = textmode ? 1 : nxTOOLS;
    if (tbox->nToolX == 0)
	/* Must be at least one tool wide */	
	tbox->nToolX = 1;
    else if (tbox->nToolX > tbox->nTool)
        tbox->nToolX = tbox->nTool;

    tbox->nToolY = tbox->nTool/tbox->nToolX;
    /* Round up */
    if (tbox->nTool % tbox->nToolX)
        ++tbox->nToolY;

    tboxUpdatePos(tbox);

    if (!tboxCreate(tbox))
        return FALSE;
    tboxShow(tbox);
    return TRUE;
}




/* Create toolbox */

tboxCreate(TBOX *tbox)
{
    if (!tboxCreateWindow(tbox))
        return FALSE;
    toolsCreate(tbox);
    return TRUE;
}



/* Create toolbox window */

tboxCreateWindow(TBOX *tbox)
{
    int x, y, width, height;

    /* this is the main dialog window; all of the controls reside within */
    tbox->hwnd = CreateWindow(
                ToolboxName,
		tbox->title,
		tbox->dwStyle,
		0, 0, 0, 0,
		tbox->hwndOwner,
		(HMENU)NULL,
		tbox->hInstance,
		NULL);

    if (tbox->hwnd == NULL) {
        adsw_msg_box(0, "Error creating toolbox window");
	return FALSE;
    }
    SetWindowLong(tbox->hwnd, 0, (long)tbox);
    return TRUE;
}

/* Reposition all locked toolboxes, typically during a WM_SIZE message */

void tboxPosAll()
{
    int index;
    TBOX *tbox;

    for (index = 0; index < ELEMENTS(tboxAry); ++index) {
        tbox = &tboxAry[index];
        if (tbox->hwnd != NULL && !(tbox->dwStyle & (DWORD)WS_POPUP)) {
            tboxUpdatePos(tbox);
            tboxShow(tbox);
	}
    }
}

/* Reposition a toolbox, typically during a WM_SIZE message */

void tboxPos(int index)
{
    TBOX *tbox;

    if (index < 0 || index >= ELEMENTS(tboxAry))
        return;
    tbox = &tboxAry[index];
    tboxUpdatePos(tbox);
    tboxShow(tbox);
}


/* Reposition a toolbox locked to the edge of a window */

tboxUpdatePos(TBOX *tbox)
{
    RECT rectO;
    int cxO, widthT, heightT;

    widthT = tboxWidth(tbox);
    heightT = tboxHeight(tbox);
    tboxAdjustWin(tbox->dwStyle, &widthT, &heightT);

    /* Position relative to upper right of owner window */
    GetClientRect(tbox->hwndOwner, &rectO);
    cxO = Width(rectO)+1;

    tbox->rect.left = cxO - widthT - tbox->right;
    tbox->rect.right = cxO - tbox->right;
    tbox->rect.top = tbox->top;
    tbox->rect.bottom = tbox->top + heightT;
    /* If its not a child, adjust position relative to the 
       parent window */
    if (tbox->dwStyle & (DWORD)WS_POPUP) {
        GetWindowRect(tbox->hwndOwner, &rectO);
	OffsetRect(&tbox->rect, rectO.left, rectO.top);
    }
}


/* Set size of toolbox frame window based on client rectangle. */

tboxAdjustWin(DWORD dwStyle, int *cx, int *cy)
{
    RECT rect;

    rect.left = 0;
    rect.top = 0;
    rect.right = *cx;
    rect.bottom = *cy;

    /* Get window rectangle for desired client rectangle.
       FALSE: no menu in window */
    AdjustWindowRect(&rect, dwStyle, FALSE);

    *cx = Width(rect)+1;
    *cy = Height(rect)+1;
}




/* Return toolbox width based on number of tools in a row
   (tbox->nToolX) */

int tboxWidth(TBOX *tbox)
{
    return (tbox->nToolX * tbox->tx + (2 * wFRAME));
}

/* Return toolbox height based on number of tools in a column
   (tbox->nToolY) */

int tboxHeight(TBOX *tbox)
{    
    int height = tbox->nToolY * tbox->ty + (2 * wFRAME);
    return height;
}

int toolWidth(TBOX *tbox)
{
    int idx, wid, slen, maxwid = 0;
    char *str;
    HDC hdc;

#if 1
    if (!tbox->flags.text)
	/* Default bitmap tool width */
        return xTOOL;
#else
    /* Make locked toolbox's wide enough for text to allow
       mixed icons and text */	
    if (tbox->dwStyle & (DWORD)WS_POPUP) 
	/* Default bitmap tool width */
        return xTOOL;
#endif
    hdc = GetDC(tbox->hwndOwner);
    for (idx = 0; idx < tbox->nTool; ++idx) {
        str = tbox->tool[idx].text;
	if (str == NULL)
            str = tbox->tool[idx].bmap_name;
	slen = strlen(str);
        wid = LOWORD(GetTextExtent(hdc, str, slen));
        if (wid > maxwid)
	    maxwid = wid;
    }		    
    ReleaseDC(tbox->hwndOwner, hdc);
    wid = max(maxwid, 2);
    return wid + wEDGE + wEDGE + 2;
}

int toolHeight(TBOX *tbox)
{
    if (!tbox->flags.text)
        return yTOOL;

    return font_ave_height + wEDGE + 2;
}

/* Remove all toolboxes */

void tboxQuitAll(void)
{
    int idx, nboxes;

    nboxes = ELEMENTS(tboxAry);
    for (idx = 0; idx < nboxes; ++idx);
	tboxQuit(idx);

    /* Restore AutoCAD's procedure */
    if (lpfnParentProc != NULL) {
	SetWindowLong(adsw_hwndAcad, GWL_WNDPROC, (LONG)lpfnParentProc);
	lpfnParentProc = NULL;
    }
}

/* Remove a toolbox */

void tboxQuit(int index)
{
    if (index < 0 || index >= ELEMENTS(tboxAry))
        return;
    if (tboxAry[index].hwnd != (HWND)0) {
        DestroyWindow(tboxAry[index].hwnd);
	tboxAry[index].hwnd = (HWND)0;
    }
}

/* Fill in rectangle data for tools */

toolsCreate(TBOX *tbox)
{
    int x, y, left, top, idx = 0;
    TOOL *tool;
    int tx = tbox->tx;
    int ty = tbox->ty;

    for (y = 0; y < tbox->nToolY; ++y) {
        for (x = 0; x < tbox->nToolX; ++x) {
	    /* Check for incomplete last row */
	    if (idx >= tbox->nTool)
	        break;
	    tool = &tbox->tool[idx];
	    left = tool->rect.left = x * tx + wFRAME;
	    tool->rect.right = left + tx;
	    top = tool->rect.top = y * ty + wFRAME;
	    tool->rect.bottom = top + ty;
	    ++idx;
	}
    }
}

/* Show all the toolbox's (generate WM_PAINT msg). */

void tboxShowAll(void)
{
    int index;
    TBOX *tbox;

    for (index = 0; index < ELEMENTS(tboxAry); ++index) {
        tbox = &tboxAry[index];
        UpdateWindow(tbox->hwnd);
    }	
    if (tbox->hwndOwner != NULL)
        BringWindowToTop(tbox->hwndOwner);
}


/* Show the toolbox in relation to the main window */

void tboxShow(TBOX *tbox)
{
    int x, y, cx, cy;

    /* Position toolbox */
    x = tbox->rect.left;
    y = tbox->rect.top;
    cx =  Width(tbox->rect);
    cy =  Height(tbox->rect);

    SetWindowPos(tbox->hwnd, HWND_TOP, x, y, cx, cy, SWP_NOACTIVATE);
				      /* Some SWP_ parameters inhibit
				         positioning */
    ShowWindow(tbox->hwnd, SW_SHOW);
    /* generate the WM_PAINT msg */
    UpdateWindow(tbox->hwnd);
    if (tbox->hwndOwner != NULL)
        BringWindowToTop(tbox->hwndOwner);
}



/*

		T o o l b o x     P r o c e d u r e

		Windows callback function for toolbox

*/

long FAR PASCAL tboxProc(HWND hwnd, WORD msg, WORD wParam, LONG lParam)
{
    TBOX *tbox;
    HDC hdc;
    PAINTSTRUCT paintst;
    RECT *rect;
    int toolidx, index;
    static int lastButton;

    switch (msg) {
    case WM_LBUTTONUP:
    case WM_LBUTTONDOWN:
        hdc = GetDC(hwnd);
        tbox = (TBOX *)GetWindowLong(hwnd, 0);
  	toolidx = toolHitMsg(tbox, lParam);
	if (msg == WM_LBUTTONUP) {
	    /* Un-inverse the original toolidx */
	    shadeRect(hdc, &tbox->tool[lastButton].rect, 0, FALSE);
	    if (toolidx == lastButton && toolidx >= 0) {
	        char *str;
		HDC hdcOwner;

	        CurrentTool = toolidx;
	        /* Send toolidx's command */
		toolText(tbox, toolidx, TRUE);
	        /* Send button's command to AutoCAD */
	        sendAcad(tbox->tool[toolidx].command);
	    }
	    /* Give focus back to main window */
            BringWindowToTop(adsw_hwndAcad);
	    ReleaseCapture();
	} else {
	    /* Button DOWN */
	    if (toolidx >= 0) {
	        /* We don't want to leave the button inverted if the mouse
	           moves away from the button. */
	        SetCapture(hwnd);
	        /* Inverse the button */
	        shadeRect(hdc, &tbox->tool[toolidx].rect, 0, TRUE);
		lastButton = toolidx;
	    }
	}
	ReleaseDC(hwnd, hdc);
        return 0 ;

    case WM_MOUSEMOVE:
        tbox = (TBOX *)GetWindowLong(hwnd, 0);
  	toolidx = toolHitMsg(tbox, lParam);
	if (toolidx < 0)
	    break;
	toolText(tbox, toolidx, FALSE);
        break;

    case WM_PAINT:
	hdc = BeginPaint(hwnd, &paintst);
        tbox = (TBOX *)GetWindowLong(hwnd, 0);
	toolsDraw(hdc, tbox);
   	EndPaint(hwnd, &paintst);
        return 0;

    case WM_CLOSE:
        tboxQuitAll();
	break;

    case WM_DESTROY:
	index = tboxFind(hwnd);
	if (index >= 0)
            tboxAry[index].hwnd = (HWND)0;
	break;

    }
    return DefWindowProc(hwnd, msg, wParam, lParam);
}


/*
		AutoCAD Subclassed Procedure

		Pick up AutoCAD's messages and pass them
		on to AutoCAD after looking at a few.
*/

long FAR PASCAL
ParentSubProc(HWND hwnd, unsigned msg, WORD wParam, LONG lParam)
{
    switch (msg) {
    /* We want to reposition the tool palette when the parent
       changes size.  We wait for WM_PAINT so that the size
       is in effect. */
    case WM_PAINT:
	tboxPosAll();
	break;

    default:
        break;
    }

    /* Pass the message to AutoCAD */
    return CallWindowProc(lpfnParentProc, hwnd, msg, wParam, lParam);
}



int tboxFind(HWND hwnd)
{
    int idx, nboxes = ELEMENTS(tboxAry);

    for (idx = 0; idx < nboxes; ++idx)
        if (hwnd == tboxAry[idx].hwnd)
	    return idx;
    return -1;
}



int toolsDraw(HDC hdc, TBOX *tbox)
{
    HDC hMemDC = (HDC)0;  	      /* handle to memory device context */
    TOOL *tool;
    int idx, nb, x1, y1, x2, y2, button;
    RECT *rect;
    int prevmode;
    int text = tbox->flags.text;

    nb = tbox->nTool;
    prevmode = SetBkMode(hdc, TRANSPARENT);
    if (!text) {
	hMemDC = CreateCompatibleDC(hdc);
        if (hMemDC == (HDC)0)
            return FALSE;
    }
    /* Draw images */
    for (idx = 0; idx < nb; ++idx) {
        rect = &tbox->tool[idx].rect;
        /* Give it 3-d look */
        shadeRect(hdc, rect, wEDGE, FALSE);
	if (text) {
            if (!toolDrawText(hdc, tbox, &tbox->tool[idx]))
                break;
	} else {
            if (!toolBlit(hdc, hMemDC, tbox, &tbox->tool[idx]))
                if (!toolDrawText(hdc, tbox, &tbox->tool[idx]))
                    break;
	}
    }
#if USE_LINE_BORDER
    {
    int twdith = tbox->tx;
    int ty = tbox->ty;

    /* Draw lines between buttons */
    for (idx = 1; idx < tbox->nToolX; ++idx) {
        int x1, y2;
        x1 = idx * tx + wFRAME;
        y2 = tbox->nToolY * ty + wFRAME;
        MoveTo(hdc, x1, 0);
        LineTo(hdc, x1, y2);
    }
    for (idx = 1; idx < tbox->nToolY; ++idx) {
        int x2, y1;
        y1 = idx * ty + wFRAME;
        x2 = tbox->nToolX * tx + wFRAME;
        MoveTo(hdc, 0, y1);
        LineTo(hdc, x2, y1);
    }
    }
#endif
    if (!text)
        DeleteDC(hMemDC);
    return TRUE;
}


int toolDrawText(HDC hdc, TBOX *tbox, TOOL *tool)
{
    char *str = tool->text;
    if (str == NULL)
        str = tool->bmap_name;

    TextOut(hdc, tool->rect.left+wEDGE+1, 
                 tool->rect.top+wEDGE-1, 
		 str, strlen(str));
    return TRUE;
}


/* Load bitmap from resource, blit into certain rectangle of tbox,
   free bitmap */

int toolBlit(HDC hDC, HDC hMemDC, TBOX *tbox, TOOL *tool)
{
    BITMAP Bitmap;                    /* bitmap structure */
    HBITMAP hBitmap;
    int stat;
    HANDLE prevObj;
    static int loaderr = FALSE;

    hBitmap = LoadBitmap(tbox->hInstance, tool->bmap_name);
    if (hBitmap == (HBITMAP)0) {
#ifdef DISP_ERR
        if (!loaderr) {
            char msg[80];
            sprintf(msg, "Error loading bitmap %s", tool->bmap_name);
            adsw_msg_box(0, msg);
            loaderr = TRUE;
        }
#endif
        return FALSE;
    }

    stat = GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&Bitmap);
    if (stat == 0)
        return FALSE;

    prevObj = SelectObject(hMemDC, hBitmap);
    BitBlt(hDC, tool->rect.left+wEDGE+wEXTRA/2, tool->rect.top+wEDGE,
                Bitmap.bmWidth, Bitmap.bmHeight, hMemDC, 0, 0, SRCCOPY);

    SelectObject(hMemDC, prevObj);
    DeleteObject(hBitmap);
    return TRUE;
}

#ifdef TBTEST

#define OPT_BLT 0

/* Load bitmap from resource, stretch blit into window client area,
   free bitmap */

int imageToWin(HWND hwnd, HDC hdc, PAINTSTRUCT *paintst, 
				HANDLE hInstance, char *bmap_name)
{
    BITMAP Bitmap;                    /* bitmap structure */
    HBITMAP hBitmap;
    int stat;
    HANDLE prevObj;
    static int loaderr = FALSE;
    HDC hMemDC;
    RECT rect;
#if OPT_BLT
    HDC shMemDC;
    RECT drect;
#endif

    hMemDC = CreateCompatibleDC(hdc);
    if (hMemDC == (HDC)0)
        return FALSE;
    hBitmap = LoadBitmap(hInstance, bmap_name);
    if (hBitmap == (HBITMAP)0) {
        if (!loaderr) {
            char msg[80];
            sprintf(msg, "Error loading bitmap %s", bmap_name);
            adsw_msg_box(0, msg);
            loaderr = TRUE;
    	    DeleteDC(hMemDC);
        }
        return FALSE;
    }
#if OPT_BLT
    shMemDC = CreateCompatibleDC(hdc);
    if (shMemDC == (HDC)0)
        return FALSE;
#endif

    stat = GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&Bitmap);
    if (stat == 0)
        return FALSE;

    prevObj = SelectObject(hMemDC, hBitmap);
    GetClientRect(hwnd, &rect);

#if OPT_BLT
    CopyRect(&drect, &paintst->rcPaint);

    /* Stretch bitmap to widow size memory hdc. */
    StretchBlt( shMemDC, 
    		rect.left, rect.top, Width(rect), Height(rect), 
		hMemDC, 
		0, 0, Bitmap.bmWidth, Bitmap.bmHeight,
		SRCCOPY);

    /* Blit only the damaged rectangle to the screen */
    BitBlt(hdc, 
    	   drect.left, drect.top, Width(drect), Height(drect), 
           shMemDC, 
	   0, 0, 
	   SRCCOPY);
#else
    StretchBlt( hdc, 
    		rect.left, rect.top, Width(rect), Height(rect), 
		hMemDC, 
		0, 0, Bitmap.bmWidth, Bitmap.bmHeight,
		SRCCOPY);
#endif

#if OPT_BLT
    DeleteDC(shMemDC);
#endif
    /* Unhook hbitmap */
    SelectObject(hMemDC, prevObj);
    DeleteObject(hBitmap);
    DeleteDC(hMemDC);
    return TRUE;
}

#endif  /* TBTEST */

/* Give button a Windows style 3-D look using white, black, light
   gray, and dark gray.  Use the SDK zoom tool to study the exact
   button edge colors and placement.  Set eEdge (extra edge) higher
   to build a wider than normal button edge that will work inward. */
void shadeRect(HDC hdc, RECT *rect, int eEdge, int inverse)
{
    HANDLE prevObj;
    HPEN gray_pen;
    int idx;

    if (eEdge >= wEDGEDEF)
        eEdge -= wEDGEDEF;
    prevObj = SelectObject(hdc, GetStockObject(BLACK_PEN));

    MoveTo(hdc, rect->left+wBORDER, rect->bottom-wBORDER-1);
    LineTo(hdc, rect->left+wBORDER, rect->top+wBORDER);
    MoveTo(hdc, rect->left+wBORDER+1, rect->top+wBORDER);
    LineTo(hdc, rect->right-wBORDER, rect->top+wBORDER);

    MoveTo(hdc, rect->right-wBORDER, rect->top+wBORDER+1);
    LineTo(hdc, rect->right-wBORDER, rect->bottom-wBORDER);
    MoveTo(hdc, rect->right-wBORDER-1, rect->bottom-wBORDER);
    LineTo(hdc, rect->left+wBORDER, rect->bottom-wBORDER);

    gray_pen = CreatePen(PS_SOLID, 1, RGB_DKGRAY);
    if (inverse)
        SelectObject(hdc, gray_pen);
    else
        SelectObject(hdc, GetStockObject(WHITE_PEN));
    for (idx = wBORDER+1; idx <= wBORDER+1+eEdge; ++idx) {
    	MoveTo(hdc, rect->left+idx, rect->bottom-idx-1);
    	LineTo(hdc, rect->left+idx, rect->top+idx);
    	LineTo(hdc, rect->right-idx, rect->top+idx);
    }

    if (inverse)
        DeleteObject(SelectObject(hdc, GetStockObject(WHITE_PEN)));
    else
        SelectObject(hdc, gray_pen);
    for (idx = wBORDER+1; idx <= wBORDER+2+eEdge; ++idx) {
    	MoveTo(hdc, rect->right-idx, rect->top+idx);
    	LineTo(hdc, rect->right-idx, rect->bottom-idx);
    	LineTo(hdc, rect->left+idx-1, rect->bottom-idx);
    }
    SelectObject(hdc, prevObj);
    if (!inverse)
        DeleteObject(gray_pen);
}

int toolHitMsg(TBOX *tbox, LONG lParam) 
{
    RECT *rect;
    POINT pt;
    int toolidx;

    pt.x = LOWORD(lParam);
    pt.y = HIWORD(lParam);
    toolidx = toolHit(tbox, &pt);
    return toolidx;
}

/* Return index of button pressed with mouse up */

int toolHit(TBOX *tbox, POINT *pt)
{
    int idx;
    RECT *rect;

    int nb = tbox->nTool;
    for (idx = 0; idx < nb; ++idx) {
        rect = &tbox->tool[idx].rect;
        if (rectHit(rect, pt))
	    return idx;
    }
    return -1;
}

/* Is this point within this rectangle? */

int rectHit(RECT *rect, POINT *pt)
{
    if (    pt->x < rect->left   ||
	    pt->y < rect->top    ||
	    pt->x > rect->right  ||
	    pt->y > rect->bottom)
	return FALSE;
    else
        return TRUE;
}



void toolText(TBOX *tbox, int index, int select)
{
    char *str;
    char text[120];

    // str = tbox->tool[index].command;
    str = tbox->tool[index].text;
    if (str == NULL)
        str = tbox->tool[index].bmap_name;
    if (select)
        strcpy(text, "* * * ");
    else
        text[0] = 0;
    strcat(text, str);
    if (select)
        strcat(text, " * * *");
    if (tbox->dwStyle & (DWORD)WS_POPUP) 
        SetWindowText(tbox->hwnd, text);
#ifdef USE_TITLE_BAR
    else if (!tbox->flags.text)
        /* Icon mode, no title bar in toolbox, put help text in
           main window title bar */
        SetWindowText(adsw_hwndAcad, text);
#endif

    /*
     * hdcOwner = GetDC(tbox->hwndOwner);
     * TextOut(hdcOwner, 0, 0, str, strlen(str));
     * ReleaseDC(tbox->hwndOwner, hdcOwner);
     */
}




/************************************************************************/
/*                                                                      */
/* Dialog Window Procedure                                              */
/*                                                                      */
/* This procedure is associated with the dialog box that is included in */
/* the function name of the procedure. It provides the service routines */
/* for the events (messages) that occur because the end user operates   */
/* one of the dialog box's buttons, entry fields, or controls.          */
/*                                                                      */
/************************************************************************/

BOOL FAR PASCAL ATBOXDLGMsgProc(HWND hWndDlg, WORD Message, WORD wParam,
                                        LONG lParam)
{
    static struct {
        unsigned floating : 1;
        unsigned enabled : 1;
    } dflags;

    switch(Message) {
    case WM_INITDIALOG:
        /* initialize working variables                                */
        /* for (idx = 0; idx < tbox->nTool; ++idx) {
         *     if (tbox[idx]->hwnd != (HWND)0)
         * }
         */
        dflags.floating = dflags.enabled = 0;
        if (tboxAry[0].hwnd != (HWND)0) {
            CheckDlgButton(hWndDlg, IDD_TBOX_ENABLE, TRUE);
        }
        CheckRadioButton(hWndDlg, IDD_FLOAT, IDD_FIXED,
                tboxAry[0].dwStyle == styleAry[TB_FLOAT] ? 
			IDD_FLOAT : IDD_FIXED);
        break;                        /* End of WM_INITDIALOG          */

    case WM_CLOSE:
        /* Closing the Dialog behaves the same as Cancel               */
        PostMessage(hWndDlg, WM_COMMAND, IDCANCEL, 0L);
        break;                        /* End of WM_CLOSE                                      */

    case WM_COMMAND:
        switch(wParam) {
        case IDD_TBOX_ENABLE:         /* Checkbox text: "Enabled"   */
            dflags.enabled = TRUE;
            break;

        case IDD_FLOAT:               /* Radio text: "Floating"   */
            dflags.floating = TRUE;
            break;

        case IDD_FIXED:               /* Radio text: "Fixed to Right"   */
            dflags.floating = FALSE;
            break;

        case IDCANCEL:
            /* Ignore data values entered into the controls        */
            /* and dismiss the dialog window returning FALSE       */
            EndDialog(hWndDlg, FALSE);
            break;

        case IDOK:
            if (IsDlgButtonChecked(hWndDlg, IDD_TBOX_ENABLE)) {
                if (IsDlgButtonChecked(hWndDlg, IDD_FLOAT))
                    tboxFloat(adsw_hwndAcad, 0);
		else
	            /* Box 0 locked to right, 2 icons wide */	
    	            tboxDraw(adsw_hwndAcad, 0, 0, 0, 2, 25, 0, FALSE);
            } else
                tboxQuit(0);

            EndDialog(hWndDlg, TRUE);
            break;
        }
        break;                        /* End of WM_COMMAND                                 */

    default:
        return FALSE;
    }
    return TRUE;
}                                     /* End of ATBOXDLGMsgProc                                      */


/* end of file */


