/*

    WINBLANK.C

    A Windows/ADS Application Starting Point

    This is to be used as a starting point, or skeleton, for new
    Windows ADS programs using Windows features such as dialogs.
    Substitute new functions for "sample" and "sample2" below.

    Commands:

	SAMPLE		Displays message in the AutoCAD text window.
	SAMPLE2 	List all blocks with constant attributes


*/

#include <windows.h>
#include "adslib.h"    

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>

#include "windde.h"		      /* a few convenient globals */	


/* ADS/AutoCAD command list structure */
typedef struct {
    char *cmdname;
    void (*cmdfunc)(void);
} CMDTAB;
#define ELEMENTS(array) (sizeof(array)/sizeof((array)[0]))

static WORD wm_acad;


/* Function Prototypes for winblank.c */

void main(int argc, char **argv);
static short funcload(void);
static void sample(void);
static void sample2(void);


/* Table of AutoCAD function (or command) names
   and corresponding C function name 
*/
CMDTAB cmdtab[] = {
    /*	 Name	 Function  */
    {"C:sample", sample},   
    {"C:sample2", sample2},   
};



/* 

	Example Dialog Data


*/

#define NAMELEN 72
#define PROF_FILE "winblank.ini"
#define PROF_MAIN "winblank"
#define PROF_TEXT "text"
#define PROF_TOGGLE "toggle"

typedef struct {
    char text[NAMELEN+1];
    int toggle;
} sGlobalData;

sGlobalData GlobalData = { "", 0 };

#define IDD_TEXT 101
#define IDD_TOGGLE 102
#define DLGNAME "DIALOG_1"

int winblank_dialog(void);
BOOL FAR PASCAL WINDLGMsgProc(HWND hWndDlg, WORD Message, WORD wParam, 
	LONG lParam);
void WriteProfile(char *profFile);
int GetProfile(char *profFile);




/*

	WINBLANK main

*/



void main(int argc, char **argv)
{
    short scode = RSRSLT;
    short cindex;
    short ads_req; 

    ads_init(argc, argv); 

    /* Get special message with which to send AutoCAD commands */
    wm_acad = RegisterWindowMessage("WM_ACAD");
    /* To use SendMessage to send a command to AutoCAD...
       SendMessage(adsw_hwndAcad, wm_acad, 0, (LONG)(LPBYTE)"cmdecho 0 "); */

#ifdef SHOW_AS_ICON
    /* Display ADS window as icon */
    ShowWindow(adsw_hWnd, SW_SHOWMINNOACTIVE);
    UpdateWindow(adsw_hWnd);
#endif

    /* Get data stored by sample dialog */
    GetProfile(PROF_FILE);

    for ( ;; ) {

	/* This process will block (stop) here until AutoCAD has a 
	   request for us */

	ads_req = ads_link(scode);	  
	if (ads_req < 0)
	    break;

	scode = RSRSLT; 	      /* default return code */

	/* Check for AT LEAST the following cases here */
	switch (ads_req) { 

	case RQXLOAD:	       /* Load request. Send function names. */
	    scode = -(funcload() ? RSRSLT : RSERR);
	    break;

        /* Execute a "loaded" function that was defined via RQXLOAD */
	case RQSUBR:
	    cindex = ads_getfuncode();

	    if (cindex >= 0 && cindex < ELEMENTS(cmdtab))
		(*cmdtab[cindex].cmdfunc)();

	    break;

        /* (xunload "adsw_AppName") entered or called in AutoCAD */ 
	case RQXUNLD:	   
	    break;

	default:
	    break;
	}
    }		                      /* end of ADS loop */

}





/* Register our functions with the ADS system.	Also,
   any drawing initialization can go here, since this is
   called with each new drawing. 
*/
static short funcload(void)
{
    int idx;
    char *cname;

    for (idx = 0; idx < ELEMENTS(cmdtab); ++idx) {
	cname = cmdtab[idx].cmdname;
	ads_defun(cname, idx);
    }

    return TRUE;
}


static void sample(void)
{
    ads_retint(winblank_dialog());
}


static void sample2(void)
{
    ads_retstr("Enter 'sample' to run the sample dialog\n");
}


/*


	Windows ADS Sample Dialog


*/


int winblank_dialog(void)
{
    int ok;
    FARPROC lpfnWINDLGMsgProc;

    lpfnWINDLGMsgProc = MakeProcInstance((FARPROC)WINDLGMsgProc, 
		adsw_hInstance); 
    ok = DialogBox(adsw_hInstance, (LPSTR)DLGNAME, adsw_hWnd, 
    						lpfnWINDLGMsgProc);
    FreeProcInstance(lpfnWINDLGMsgProc);

    /* Set focus back to AutoCAD's graphics window */
    SetFocus(adsw_hwndAcad);

    return ok;
}



/************************************************************************/
/*									*/
/* Dialog Window Procedure						*/
/*									*/
/* This procedure is associated with the dialog box that is included in */
/* the function name of the procedure. It provides the service routines */
/* for the events (messages) that occur because the end user operates	*/
/* one of the dialog box's buttons, entry fields, or controls.          */
/*									*/
/************************************************************************/

BOOL FAR PASCAL WINDLGMsgProc(HWND hWndDlg, WORD Message, WORD wParam, 
	LONG lParam)
{ 
    static char text[NAMELEN+1];
    static int toggle;
    int chng;

    switch(Message) {

    case WM_INITDIALOG:
	 /* initialize working variables				*/
	 strcpy(text, GlobalData.text);
	 SetDlgItemText(hWndDlg, IDD_TEXT, text);
	 SendDlgItemMessage(hWndDlg, IDD_TEXT, EM_SETSEL,
			NULL, MAKELONG(0, 0x7fff));
	 CheckDlgButton(hWndDlg, IDD_TOGGLE, GlobalData.toggle);
	 toggle = GlobalData.toggle;
	 break;
			   
    case WM_CLOSE:
	 /* Closing the Dialog behaves the same as Cancel		*/		
	 PostMessage(hWndDlg, WM_COMMAND, IDCANCEL, 0L);
	 break; /* End of WM_CLOSE					*/
	   
    case WM_COMMAND:
	 switch(wParam)
	   {
	    case IDD_TEXT: /* Edit Control				*/
		/* Indicates changes have occured */
		/* if(HIWORD(lParam) == EN_CHANGE) */
		break;

            case IDD_TOGGLE: /* Checkbox text: 			*/
		 break;        

	    case IDOK:
	         chng = FALSE;
                 GetDlgItemText(hWndDlg, IDD_TEXT, text, NAMELEN);

		 if (stricmp(GlobalData.text, text)) {
		     chng = TRUE;	
		     strcpy(GlobalData.text, text);
		 }

		 toggle = IsDlgButtonChecked(hWndDlg, IDD_TOGGLE);
		 if (toggle != GlobalData.toggle) {
		     chng = TRUE;
		     GlobalData.toggle = toggle;
		 }
		 if (chng)
		     WriteProfile(PROF_FILE);
		 EndDialog(hWndDlg, TRUE);
		 break;

	    case IDCANCEL:
		 /* Ignore data values entered into the controls	*/
		 /* and dismiss the dialog window returning FALSE	*/
		 EndDialog(hWndDlg, FALSE);
		 break;
	   }
	 break;    /* End of WM_COMMAND 				*/
				  
    default:
	return FALSE;
   }
 return TRUE;
}



/* Save and restore user options from windows\WINBLANK.INI file, using
   the Windows PrivateProfile functions. */

void WriteProfile(char *profFile)
{
    char *str;

    WritePrivateProfileString(PROF_MAIN,
		PROF_TEXT, GlobalData.text, profFile);
    str = (GlobalData.toggle ? "1" : "0"); 
    WritePrivateProfileString(PROF_MAIN, PROF_TOGGLE, str, profFile);
}

int GetProfile(char *profFile)
{
    if (!GetPrivateProfileString(PROF_MAIN, PROF_TEXT, GlobalData.text, 
		GlobalData.text, NAMELEN, profFile))
	return FALSE;
    GlobalData.toggle = GetPrivateProfileInt(PROF_MAIN, PROF_TOGGLE, 1, 
    		profFile);
    return TRUE;
}


