#       Makefile for Windows AutoCAD ADS for Microsoft C 6.00AX
#
#       Use Microsoft "NMK" make utility.
#
#       "nmk -f winads.mak ads_perr"
#
# -Zi => create object file for use with CodeView
# -Od => disable code rearrangement optimization for debugger
# -AL => Large model (large code, large data) 
# -G2 => use 80286 instructions
# -Gs => turn off stack checking
# -J  => treat char as unsigned char
# -Zp => pack structure data on byte boundries
# -Zl => no default libraries
# -FPi87 => inline 80x87 code
# -Gw => Windows program
# -EM => Use extended memory, C 6.00AX.  Fixes "Out of far heap".

CF = -c -AL -EM -G2sw -FPi87 -DADS -DWIN

#Link flags
LFLAG = /NOD /NOE

#If you wish to build the sample applications with debugging information
#then set the environment variable CV to 1, i.e. set CV=1
!IFDEF CV
#CodeView Version
CFLG = $(CF) -Zip -Od -DCV
LFLG = $(LFLAG) /CO
!ELSE
#Retail Version
CFLG = $(CF) -Zp
LFLG = $(LFLAG)
!ENDIF

COMP = cl $(CFLG)


# Libraries
LIBS = winadsc6 libw ddeml llibcew

# geomcal objects
CALOBJS = \
        cal.obj      \
        calerr.obj   \
        callex.obj   \
        calexpr.obj  \
        calmngf.obj  \
        calstdf.obj  \
        calusrf.obj  \
        util.obj

all:    \
        winadsc6.obj \
        ads_perr.exe \
        appmngr.exe  \
        arbmat.exe   \
        colext.exe   \
        dlgtest.exe  \
        dragger.exe  \
        fact.exe     \
        gpalsym.exe  \
        gravity.exe  \
        grvecs.exe   \
        magnets.exe  \
        mountain.exe \
        sld2ps.exe   \
        sqr.exe      \
        tadc.exe     \
        tower.exe    \
        geomcal.exe

#Allow building one with "nmake -f winads.mak ads_perr"
ads_perr: ads_perr.exe
appmngr: appmngr.exe
arbmat: arbmat.exe
colext: colext.exe
dlgtest: dlgtest.exe
dragger: dragger.exe
fact: fact.exe
gpalsym: gpalsym.exe
gravity: gravity.exe
grvecs: grvecs.exe
magnets: magnets.exe
mountain: mountain.exe
sld2ps: sld2ps.exe
sqr: sqr.exe
tadc: tadc.exe
tower: tower.exe
geomcal: geomcal.exe

winadsc6.obj: winads.c windde.h
        $(COMP) /Fowinadsc6.obj winads.c

ads_perr.exe: ads_perr.obj winadsc6.obj winadsc6.lib
appmngr.exe: appmngr.obj winadsc6.obj winadsc6.lib
arbmat.exe: arbmat.obj winadsc6.obj winadsc6.lib
colext.exe: colext.obj winadsc6.obj winadsc6.lib
dlgtest.exe: dlgtest.obj winadsc6.obj winadsc6.lib
dragger.exe: dragger.obj winadsc6.obj winadsc6.lib
fact.exe: fact.obj winadsc6.obj winadsc6.lib
gpalsym.exe: gpalsym.obj winadsc6.obj winadsc6.lib
gravity.exe: gravity.obj winadsc6.obj winadsc6.lib
grvecs.exe: grvecs.obj winadsc6.obj winadsc6.lib
magnets.exe: magnets.obj winadsc6.obj winadsc6.lib
mountain.exe: mountain.obj winadsc6.obj winadsc6.lib
sld2ps.exe: sld2ps.obj winadsc6.obj winadsc6.lib
sqr.exe: sqr.obj winadsc6.obj winadsc6.lib
tadc.exe: sqr.obj winadsc6.obj winadsc6.lib
tower.exe: sqr.obj winadsc6.obj winadsc6.lib
geomcal.exe: $(CALOBJS) winadsc6.obj winadsc6.lib

ads_perr.obj: ads_perr.c
appmngr.obj: appmngr.c
arbmat.obj: arbmat.c
colext.obj: colext.c
dlgtest.obj: dlgtest.c
dragger.obj: dragger.c
fact.obj: fact.c
gpalsym.obj: gpalsym.c
gravity.obj: gravity.c
grvecs.obj: grvecs.c
magnets.obj: magnets.c
mountain.obj: mountain.c
sld2ps.obj: sld2ps.c
sqr.obj: sqr.c
tadc.obj: tadc.c
tower.obj: tower.c
cal.obj: cal.c
calerr.obj: calerr.c
callex.obj: callex.c
calexpr.obj: calexpr.c
calmngf.obj: calmngf.c
calstdf.obj: calstdf.c
calusrf.obj: calusrf.c
util.obj: util.c

geomcal.exe: $(CALOBJS)
        echo cal+calerr+callex+calexpr+calmngf+calstdf+calusrf+util.obj+ \
	     > geomcal.lnk
        echo winadsc6.obj >> geomcal.lnk
        link @geomcal.lnk,$*,,$(LIBS),win\$*.def $(LFLG);
        rc -tk -iwin win\$*.rc $*.exe
        del win\$*.res
        del geomcal.lnk
!IFDEF ACEXE
        copy $*.exe $(ACEXE)
!ENDIF


.SUFFIXES: .obj .exe .c

.c.obj:
        $(COMP) $*.c

.obj.exe:
        link $(LFLG) $*.obj winadsc6.obj,$*,,$(LIBS),win\$*.def;
        rc -tk -iwin win\$*.rc $*.exe
        del win\$*.res
!IFDEF ACEXE
        copy $*.exe $(ACEXE)
!ENDIF

