;;;   Julian.lsp
;;;   (C) Copyright 1988-1993 by Autodesk, Inc.
;;;  
;;;   This program is copyrighted by Autodesk, Inc. and is  licensed
;;;   to you under the following conditions.  You may not distribute
;;;   or  publish the source code of this program in any form.   You
;;;   may  incorporate this code in object form in derivative  works
;;;   provided  such  derivative  works  are  (i.) are  designed and
;;;   intended  to  work  solely  with  Autodesk, Inc. products, and
;;;   (ii.)  contain  Autodesk's  copyright  notice  "(C)  Copyright
;;;   1988-1993 by Autodesk, Inc."
;;;
;;;   AUTODESK  PROVIDES THIS PROGRAM "AS IS" AND WITH  ALL  FAULTS.
;;;   AUTODESK  SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF  MER-
;;;   CHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK,  INC.
;;;   DOES  NOT  WARRANT THAT THE OPERATION OF THE PROGRAM  WILL  BE
;;;   UNINTERRUPTED OR ERROR FREE.
;;;   
;;;   Originally designed and implemented by Kelvin R. Throop
;;;
;;;----------------------------------------------------------------------------
;;; DESCRIPTION
;;;
;;;   AutoCAD Julian date to calendar date conversion
;;;
;;;----------------------------------------------------------------------------

(defun c:date ()
	(setq td (getvar "date"))
	(setq time (* 86400.0 (- td (setq j (fix td)))))
	(setq j (- j 1721119.0))
	(setq y (fix (/ (1- (* 4 j)) 146097.0)))
	(setq j (- (* j 4.0) 1.0 (* 146097.0 y)))
	(setq d (fix (/ j 4.0)))
	(setq j (fix (/ (+ (* 4.0 d) 3.0) 1461.0)))
	(setq d (- (+ (* 4.0 d) 3.0) (* 1461.0 j)))
	(setq d (fix (/ (+ d 4.0) 4.0)))
	(setq m (fix (/ (- (* 5.0 d) 3) 153.0)))
	(setq d (- (* 5.0 d) 3.0 (* 153.0 m)))
	(setq d (fix (/ (+ d 5.0) 5.0)))
	(setq y (+ (* 100.0 y) j))
	(if (< m 10.0)
		(setq m (+ m 3))
		(progn	
			(setq m (- m 9))
			(setq y (1+ y))
		)
	)

; Now print the date. Year in Y, month in M, day in D

	(princ (fix y))
	(princ "/")
	(princ (fix m))
	(princ "/")
	(princ (fix d))

; Determine the clock time from the fraction of the day

	(setq hh (fix (/ time 3600.0)))
	(setq time (- time (* hh 3600.00)))
	(setq mm (fix (/ time 60.0)))
	(setq ss (- time (* mm 60.0)))

; Print the time

	(princ " ")
	(princ hh)
	(princ ":")
	(princ mm)
	(princ ":")
	(princ ss)
	(terpri)
)	
