;;; --------------------------------------------------------------------------;
;;; SQR.LSP
;;;   (C) Copyright 1988-1993 by Autodesk, Inc.
;;;  
;;;   This program is copyrighted by Autodesk, Inc. and is  licensed
;;;   to you under the following conditions.  You may not distribute
;;;   or  publish the source code of this program in any form.   You
;;;   may  incorporate this code in object form in derivative  works
;;;   provided  such  derivative  works  are  (i.) are  designed and
;;;   intended  to  work  solely  with  Autodesk, Inc. products, and
;;;   (ii.)  contain  Autodesk's  copyright  notice  "(C)  Copyright
;;;   1988-1993 by Autodesk, Inc."
;;;
;;;   AUTODESK  PROVIDES THIS PROGRAM "AS IS" AND WITH  ALL  FAULTS.
;;;   AUTODESK  SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF  MER-
;;;   CHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK,  INC.
;;;   DOES  NOT  WARRANT THAT THE OPERATION OF THE PROGRAM  WILL  BE
;;;   UNINTERRUPTED OR ERROR FREE.
;;;
;;; --------------------------------------------------------------------------;
;;; DESCRIPTION
;;;
;;;   This is a programming example.
;;;
;;;   This is an implementation of a square root function in
;;;   LISP using the Newton-Raphson method as used in AutoCAD.
;;;   It is intended as a test of floating point arithmetic in
;;;   our LISP, as you can check accuracy with the statement:
;;;      (- (sqr 2) (sqrt 2))
;;;   which will compare the built-in function with this one.
;;;   
;;;   John Walker  12/17/84
;;;
;;; --------------------------------------------------------------------------;

(defun sqr (x / y c cl) 
  (if (or (= 'REAL(type x)) (= 'INT(type x))) 
    (progn
      (cond ((minusp x) 'Negative-argument) 
        ((zerop x) 0.0) 
        (t (setq y (/ (+ 0.154116 (* x 1.893872)) (+ 1.0 (* x 1.047988))))
           (setq c (/ (- y (/ x y)) 2.0))
           (setq cl 0.0)
           (while (not (equal c cl)) 
             (setq y (- y c))
             (setq cl c)
             (setq c (/ (- y (/ x y)) 2.0))
           ) y
        )
      )
    ) 
    (progn
      (princ "Invalid argument.") 
      (princ)
    )
  )
) 

;;; --------------------------------------------------------------------------;

